/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : debug.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 21-01-92
 *  Notes     : debug routines
 *  Copyright : (c) Copyright 1991-92 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include "tin.h"

int debug;


void debug_print_arts ()
{
#ifdef DEBUG
	int i;

	if (! debug)
		return;

	for (i = 0; i < top; i++) {	/* for each group */
		debug_print_header (&arts[i]);
	}
#endif
}


void debug_print_header (s)
	struct article_t *s;
{
#ifdef DEBUG
	FILE *fp;

	if (! debug)
		return;

	if ((fp = fopen ("/tmp/DUMP","a+")) != NULL) {
		fprintf (fp,"art=[%5ld] killed=[%s]\n", s->artnum,
			(s->tagged ? "TRUE" : "FALSE"));
		fprintf (fp,"subj=[%-38s]\n", s->subject);
		fprintf (fp,"from=[%s]  name=[%s]\n", s->from, s->name);
 		if (s->archive) {
 		    fprintf (fp, "arch=[%-38s]  ", s->archive);
 		} else {
 		    fprintf (fp, "arch=[]  ");
 		}
 		if (s->part) {
 		    fprintf (fp, "part=[%s]  ", s->part);
 		} else {
 		    fprintf (fp, "part=[]  ");
 		}
 		if (s->patch) {
 		    fprintf (fp, "patch=[%s]\n", s->patch);
		} else {
 		    fprintf (fp, "patch=[]\n");
 		}
		fprintf (fp,"thread=[%d]  inthread=[%d]  unread=[%d]\n",
			s->thread, s->inthread, s->unread);
/*		fprintf (fp,"thread=[%s]  inthread=[%s]  unread=[%s]\n",
			(s->thread == ART_NORMAL ? "ART_NORMAL" : "ART_EXPIRED"),
			(s->inthread ? "TRUE" : "FALSE"),
			(s->unread ? "TRUE" : "FALSE"));
*/
		fflush (fp);
		fclose (fp);
		chmod ("/tmp/DUMP", 0666);
	}
#endif
}


void debug_print_comment (comment)
	char *comment;
{
#ifdef DEBUG
	FILE *fp;

	if (! debug)
		return;

	if ((fp = fopen ("/tmp/BASE","a+")) != NULL) {
		fprintf (fp,"\n%s\n\n", comment);
		fflush (fp);
		fclose (fp);
		chmod ("/tmp/BASE", 0666);
	}
#endif
}


void debug_print_base ()
{
#ifdef DEBUG
	FILE *fp;
	int i;

	if (! debug)
		return;

	if ((fp = fopen ("/tmp/BASE","a+")) != NULL) {
		for (i = 0; i < top_base; i++) {
			fprintf (fp, "base[%3d]=[%5ld]\n",i,base[i]);
		}
		fflush (fp);
		fclose (fp);
		chmod ("/tmp/BASE", 0666);
	}
#endif
}


void debug_print_active ()
{
#ifdef DEBUG
	FILE *fp;
	int i;

	if (! debug)
		return;

	if ((fp = fopen ("/tmp/ACTIVE","w")) != NULL) {
		for (i = 0; i < num_active; i++) {	/* for each group */
			fprintf (fp, "[%4d]=[%-28s] max=[%4ld] min=[%4ld] mod=[%c] nxt=[%4d] flag=[%d] read=[%d] thread=[%d]\n",
				i, active[i].name, active[i].max, active[i].min,
				active[i].moderated, active[i].next, active[i].flag,
				active[i].read, active[i].thread);
		}
		fflush (fp);
		fclose (fp);
		chmod ("/tmp/ACTIVE", 0666);
	}
#endif
}


void log_user ()
{
#ifdef LOG_USER
	char buf[33];
	FILE *fp;
	long epoch;
	extern struct passwd *myentry;

	if ((fp = fopen (LOG_USER_FILE, "a+")) != NULL) {
		time (&epoch);
		my_strncpy (buf, myentry->pw_gecos, 32);
		fprintf (fp, "%s%d: %-32s (%-8s) %s", VERSION, PATCHLEVEL,
				buf, myentry->pw_name, ctime (&epoch));
		fclose (fp);
		chmod (LOG_USER_FILE, 0666);
	}
#endif
}
