/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : init.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 04-02-92
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Iain Lea
 *              You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"


char rcdir[LEN];
char rcfile[LEN];
char indexdir[LEN];
char killfile[LEN];
char postfile[LEN];
char unthreadfile[LEN];

char active_file[LEN];
char homedir[LEN];
char userid[LEN];
char delgroups[LEN];
char newsrc[LEN];
char newnewsrc[LEN];
char add_addr[LEN];			/* address to add to rR reply to author with mail */
char bug_addr[LEN];			/* address to add send bug reports to */
char txt_help_bug_report[LEN];	/* address to add send bug reports to */
char reply_to[LEN];			/* reply-to address */
char my_org[LEN];			/* organization */
char sig[LEN];
char signature[LEN];
char default_shell_command[LEN];	/* offers user default choice */
char killsubj[LEN];			/* contains Subject:'s not to be shown */
char killfrom[LEN];			/* conatins From:'s not to be shown */
char page_header[LEN];		/* page header of pgm name and version */
char savedir[LEN];			/* directory to save articles to */
char spooldir[LEN];			/* directory where new is */
char mail_news_user[LEN];	/* mail new news to this user address */
char mailer[LEN];			/* mail program */
char maildir[LEN];			/* mailbox dir where = saves are stored */
char mailbox[LEN];			/* system mailbox for each user */
char printer[LEN];			/* printer program specified from tinrc */
char cmd_line_printer[LEN];	/* printer program specified on cmd line */
char article[LEN];			/* ~/.article file */
char dead_article[LEN];		/* ~/dead.article file */
char progname[LEN];			/* program name */
char redirect_output[LEN];	/* /dev/null or nothing if -D option */
int catchup_read_groups;	/* ask if read groups are to be marked read */
int cmd_line;				/* batch / interactive mode */
int created_rcdir;			/* checks if first time tin is started */
int default_printer;		/* set to false if user give a printer with -p switch */
int default_show_author;	/* show_author value from 'M' menu in tinrc */
int draw_arrow_mark;		/* draw -> or highlighted bar */
int full_page_scroll;		/* page half/full screen of articles/groups */
int kill_articles;			/* kill articles matching kill file contents */
int mark_saved_read;		/* mark saved article/thread as read */
int num_of_killed_files;
int num_of_tagged_files;
int pos_first_unread;		/* position cursor at first/last unread article */
int post_proc_type;			/* type of post processing to be performed */
int print_header;			/* print all of mail header or just Subject: & From lines */
int save_archive_name;		/* save thread with name from Archive-name: field */
int mail_news;				/* mail all arts to specified user */
int save_news;				/* save all arts to savedir structure */
int save_separate;			/* save a each part of a thread to a separate file */
int show_author;			/* show Subject & From or only Subject in group menu */
int show_only_unread;		/* show only new/unread arts or all arts */
int sort_art_type;			/* sort arts[] array by subject,from or date field */
int thread_arts;			/* thread/unthread articles for viewing */
struct passwd *myentry;

/*
 * init_selfinfo
 *   Deterimines users home directory, userid, and a path
 *   for an rc file in the home directory
 */

void init_selfinfo ()
{
	extern char *getenv ();
	extern char *getlogin ();
	extern struct passwd *getpwuid ();
	extern struct passwd *getpwnam ();
	char nam[LEN];
	char *p;
	FILE *fp;
	struct stat sb;

	myentry = (struct passwd *) 0;
	if ((p = getlogin ()) != (char *) 0) {
		myentry = getpwnam (p);
	} else {
		myentry = getpwuid (getuid ());
	}

	strcpy (userid, myentry->pw_name);

	if ((p = getenv ("HOME")) != NULL) {
		strcpy (homedir, p);
	} else {
		strcpy (homedir, myentry->pw_dir);
	}

	catchup_read_groups = TRUE;
	created_rcdir = FALSE;
#ifdef USE_INVERSE_HACK
	draw_arrow_mark = TRUE;
#else
	draw_arrow_mark = FALSE;
#endif
#ifdef BSD
	default_printer = TRUE;
#else	
	default_printer = FALSE;
#endif
	full_page_scroll = TRUE;
	kill_articles = FALSE;
	mark_saved_read = TRUE;
	num_of_killed_files = 0;
	num_of_tagged_files = 0;
	post_proc_type = POST_PROC_NONE;
	pos_first_unread = TRUE;
	print_header = FALSE;
	save_separate = TRUE;
	save_archive_name = TRUE;
	default_show_author = SHOW_FROM_NAME;
	show_only_unread = FALSE;
	sort_art_type = SORT_BY_NOTHING;
	save_news = FALSE;
	thread_arts = TRUE;
	
	killsubj[0] = '\0';
	killfrom[0] = '\0';
	
	nntp_server[0] = '\0';
	author_search_string[0] = '\0';
	group_search_string[0] = '\0';
	subject_search_string[0] = '\0';
	art_search_string[0] = '\0';
	default_mail_address[0] = '\0';
	default_regex_pattern[0] = '\0';
	default_save_file[0] = '\0';
	default_shell_command[0] = '\0';
	default_post_subject[0] = '\0';
	cmd_line_printer[0] = '\0';
	
	sprintf (rcdir,  "%s/%s", get_val ("TINDIR", homedir), RCDIR);
	sprintf (indexdir, "%s/%s", rcdir, INDEXDIR);
	sprintf (rcfile,   "%s/%s", rcdir, RCFILE);
	sprintf (killfile, "%s/%s", rcdir, KILLFILE);
	sprintf (postfile, "%s/%s", rcdir, POSTFILE);
	sprintf (unthreadfile, "%s/%s", rcdir, UNTHREADFILE);

	sprintf (signature, "%s/.signature", homedir);
	sprintf (sig, "%s/.Sig", homedir);
	sprintf (newsrc, "%s/.newsrc", homedir);
	sprintf (newnewsrc, "%s/.newnewsrc", homedir);
	sprintf (delgroups, "%s/.delgroups", homedir);
	sprintf (active_file, "%s/active", LIBDIR);
	sprintf (savedir, "%s/News", homedir);
	sprintf (maildir, "%s/%s", homedir, DEFAULT_MAILDIR);
	sprintf (mailbox, "%s/%s", DEFAULT_MAILBOX, userid);
	sprintf (dead_article, "%s/dead.article", homedir);
	sprintf (article, "%s/.article", homedir);

	strcpy (mailer, DEFAULT_MAILER);
	strcpy (printer, DEFAULT_PRINTER);
	strcpy (spooldir, SPOOLDIR);
	strcpy (bug_addr, BUG_REPORT_ADDRESS);
	strcpy (redirect_output, "> /dev/null 2>&1");

	set_real_uid_gid ();
	
	if (stat (rcdir, &sb) == -1) {
		created_rcdir = TRUE;
		mkdir (rcdir, 0755);
	}
	if (stat (indexdir, &sb) == -1) {
		mkdir (indexdir, 0755);
	}
	if (stat (postfile, &sb) == -1) {
		if ((fp = fopen (postfile, "w")) != NULL) {
			fclose (fp);
		}
	}
	if (stat (unthreadfile, &sb) == -1) {
		if ((fp = fopen (unthreadfile, "w")) != NULL) {
			fclose (fp);
		}
	}

	read_rcfile ();

	show_author = default_show_author;

	if (stat (savedir, &sb) == -1) {
		mkdir (savedir, 0755);
	}
	if (stat (maildir, &sb) == -1) {
		mkdir (maildir, 0755);
	}
	if (stat (active_file, &sb) >= 0)
		goto got_active;

/*
 *  I hate forgetting to define LIBDIR correctly.  Guess a
 *  couple of likely places if it's not where LIBDIR says it is.
 */

	strcpy (active_file, "/usr/lib/news/active");
	if (stat (active_file, &sb) >= 0)
		goto got_active;

	strcpy (active_file, "/usr/local/lib/news/active");
	if (stat (active_file, &sb) >= 0)
		goto got_active;

	strcpy (active_file, "/usr/public/lib/news/active");
	if (stat (active_file, &sb) >= 0)
		goto got_active;

/*
 *  Oh well. Revert to what LIBDIR says it is to produce a
 *  useful error message when read_active () fails later.
 */

	sprintf (active_file, "%s/active", LIBDIR);

got_active:

/*
 *  check enviroment for organization
 */
	my_org[0] = '\0';
	if ((p = getenv ("ORGANIZATION")) != NULL) {
		my_strncpy (my_org, p, LEN);
		goto got_org;
	}

/*
 *  check ~/.org for organization
 */
	sprintf (nam, "%s/organization", rcdir);
	fp = fopen (nam, "r");

/*
 *  check LIBDIR/organization for system wide organization
 */
	if (fp == NULL) {
		sprintf (nam, "%s/organization", LIBDIR);
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/usr/lib/news/organization");
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/usr/local/lib/news/organization");
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/usr/public/lib/news/organization");
		fp = fopen (nam, "r");
	}

	if (fp == NULL) {
		sprintf (nam, "/etc/organization");
		fp = fopen (nam, "r");
	}

	if (fp != NULL) {
		if (fgets (my_org, sizeof (my_org), fp) != NULL) {
			for (p = my_org; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_org:;

/*
 *  check enviroment for REPLYTO
 */
	reply_to[0] = '\0';
	if ((p = getenv ("REPLYTO")) != NULL) {
		my_strncpy (reply_to, p, LEN);
		goto got_reply;
	}

	sprintf (nam, "%s/replyto", rcdir);
	if ((fp = fopen (nam, "r")) != NULL) {
		if (fgets (reply_to, sizeof (reply_to), fp) != NULL) {
			for (p = reply_to; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_reply:;

/*
 *  check enviroment for ADD_ADDRESS
 */
	add_addr[0] = '\0';
	if ((p = getenv ("ADD_ADDRESS")) != NULL) {
		my_strncpy (add_addr, p, LEN);
		goto got_add_addr;
	}

	sprintf (nam, "%s/add_address", rcdir);
	if ((fp = fopen (nam, "r")) != NULL) {
		if (fgets (add_addr, sizeof (add_addr), fp) != NULL) {
			for (p = add_addr; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_add_addr:;

/*
 *  check enviroment for BUG_ADDRESS
 */
	if ((p = getenv ("BUG_ADDRESS")) != NULL) {
		my_strncpy (bug_addr, p, LEN);
		goto got_bug_addr;
	}

	sprintf (nam, "%s/bug_address", rcdir);
	if ((fp = fopen (nam, "r")) != NULL) {
		if (fgets (bug_addr, sizeof (bug_addr), fp) != NULL) {
			for (p = bug_addr; *p && *p != '\n'; p++)
				continue;
			*p = '\0';
		}
		fclose (fp);
	}

got_bug_addr:;
	sprintf (txt_help_bug_report, txt_help_bug, bug_addr);

	set_tin_uid_gid ();
}
