/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : main.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 30-01-92
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

char cvers[LEN];
char nntp_server[LEN];

int NOTESLINES;						/* set in set_win_size () */
int RIGHT_POS;						/* set in set_win_size () */
int MORE_POS;						/* set in set_win_size () */
int max_subj = 0;
int max_from = 0;
int max_active;
int group_hash[TABLE_SIZE];			/* group name --> active[] */
int num_active;						/* one past top of active */
int group_top;						/* one past top of my_group */
int catchup = FALSE;				/* mark all arts read in all subscribed groups */
int update = FALSE;					/* update index files only mode */
int update_fork = FALSE;			/* update index files by forked tin -u */
int verbose = FALSE;				/* update index files only mode */
int start_line_offset;				/* used by invoke_editor for line no. */
int read_news_via_nntp = FALSE;		/* read news locally or via NNTP */
int max_art;
int top = 0;
int top_base;
int tin_uid;
int tin_gid;
int real_uid;
int real_gid;
int local_index;					/* do private indexing? */


void main (argc, argv)
	int argc;	
	char *argv[];
{
	extern int optind;
	extern char *optarg;
	int c, i, created;
	int check_any_unread = FALSE;
	int notify_new_groups = FALSE;
	int start_any_unread = FALSE;
	int start_groupnum = 0;
	struct stat st;
	
	cmd_line = TRUE;
	debug = FALSE;

	set_signal_handlers ();

	basename (argv[0], progname);

	sprintf (page_header, "%s %s PL%d%s", progname, VERSION, PATCHLEVEL, is_remote ()); 	
	sprintf (cvers, "%s (c) Copyright 1991-92 Iain Lea.", page_header);

	if (progname[0] == 'r') {	/* rtin so read news remotely via NNTP */
#ifdef USE_NNTP			
		read_news_via_nntp = TRUE;
#else
		error_message (txt_nntp_not_enabled, progname);
		exit (1);
#endif
	}

	tin_uid = geteuid ();
	tin_gid = getegid ();
	real_uid = getuid ();
	real_gid = getgid ();

	/*
	 * we're setuid, so index in /usr/spool/news unless user root
	 */
	if (tin_uid != real_uid && real_uid != 0) {
		local_index = FALSE;
	} else {	/* index in users home directory ~/.tin/.index */
		local_index = TRUE;
	}

	init_selfinfo ();	/* set up char *'s: homedir, newsrc, etc. */
	init_alloc ();		/* allocate initial array sizes */

	while ((c = getopt (argc, argv, "cd:Df:hHm:M:np:rRs:SuUvzZ")) != EOF) {
		switch(c) {
			case 'c':
				catchup = TRUE;
				update = TRUE;
				break;
				
			case 'd':
				my_strncpy (savedir, optarg, LEN);
				set_real_uid_gid ();
				if (stat (savedir, &st) == -1) {
					mkdir (savedir, 0755);
				}
				set_tin_uid_gid ();
				break;

			case 'D':		/* debug mode */
				redirect_output[0] = '\0';
				debug = TRUE;
				break;

			case 'f':
				my_strncpy (newsrc, optarg, LEN);
				break;

			case 'H':
				show_intro_page ();
				exit (1);
				break;

			case 'm':
				my_strncpy (maildir, optarg, LEN);
				set_real_uid_gid ();
				if (stat (maildir, &st) == -1) {
					mkdir (maildir, 0755);
				}
				set_tin_uid_gid ();
				break;

			case 'M':	/* mail new news to specified user */
				my_strncpy (mail_news_user, optarg, LEN);
				mail_news = TRUE;
				update = TRUE;
				catchup = TRUE;
				break;

			case 'n':
				notify_new_groups = TRUE;
				break;

			case 'p':
				my_strncpy (cmd_line_printer, optarg, LEN);
				default_printer = FALSE;
				break;

			case 'r':	/* read news remotely from default NNTP server */
#ifdef USE_NNTP			
				read_news_via_nntp = TRUE;
#else
				error_message (txt_recompile_with_nntp_option, progname);
				exit (1);
#endif
				break;

			case 'R':	/* read news saved by -S option */
				error_message ("%s: -R option not yet implemented.", progname);
				exit (1);
				break;

			case 's':
				my_strncpy (spooldir, optarg, LEN);
				break;

			case 'S':	/* save new news to dir structure */
				save_news = TRUE;
				update = TRUE;
				break;

			case 'u':
				update = TRUE;
				break;

			case 'U':
				update_fork = TRUE;
				update = TRUE;
				break;

			case 'v':
				verbose = TRUE;
				break;

			case 'z':
				start_any_unread = TRUE;
				update = TRUE;
				break;

			case 'Z':
				check_any_unread = TRUE;
				update = TRUE;
				break;

			case 'h':
			case '?':
			default:
				usage (progname);
				exit (1);
		}
	}

	if (! update) {
		printf ("%s\n", cvers);
		fflush (stdout);
	}

	hash_init ();
	for (i = 0; i < TABLE_SIZE; i++) {
		group_hash[i] = -1;
	}

	nntp_startup ();				/* if -r connect to nntp server */
	created = read_active ();		/* load the active file into active[] */

	if (optind < argc) {
		while (optind < argc) {
			if (add_group (argv[optind], TRUE) < 0) {
				error_message (txt_not_in_active_file, argv[optind]);
			}
			optind++;
		}
	} else {
		backup_newsrc ();
		read_newsrc (TRUE);
		mark_unthreaded_groups ();
	}

	if (kill_articles) {		/* read in users kill file */
		read_kill_file ();
	}
	
	/*
	 *  log name of user and start time to central logfile
	 */
	log_user ();

	/*
	 *  check/start if any new/unread articles
	 */
	start_groupnum = check_for_any_new_news (check_any_unread, start_any_unread);

	/*
	 *  mail any new articles to specified user
	 *  or
	 *  save any new articles to savedir structure for later reading
	 */
	save_or_mail_new_news ();
	
	/*
	 *  update index files
	 */
	update_index_files ();
	
	if (! InitScreen ()) {
		error_message (txt_screen_init_failed, progname);
		exit (1);
	}

	cmd_line = FALSE;
	ScreenSize (&LINES, &COLS);		/* screen size from termcap entry */
	Raw (TRUE);

	set_win_size (&LINES, &COLS);	/* screen size doing ioctl() call */

	if (notify_new_groups && ! created) {	/* check for new groups */
		notify_groups ();
	}

	if (created_rcdir && update == FALSE) {
		show_intro_page (FALSE);		
	}
	
#ifdef READ_CHAR_HACK
	setbuf (stdin, 0);
#endif

	selection_index (start_groupnum);

	tin_done (0);
}

/*
 * usage
 */

void usage (progname)
	char *progname;
{
	error_message ("%s A threaded Netnews reader.\n", cvers);
	error_message ("Usage: %s [options] [newsgroups]", progname);
	error_message ("  -c       mark all news as read in subscribed newsgroups (batch mode)", (char *) 0);
	error_message ("  -d dir   directory to use when saving news [default=%s]", savedir);
	error_message ("  -f file  subscribed to newsgroups file [default=%s]", newsrc);
	error_message ("  -h       help", (char *) 0);
	error_message ("  -H       help information about %s", progname);
	error_message ("  -m dir   mailbox directory [default=%s]", maildir);
	error_message ("  -M user  mail new news to specified user (batch mode)", (char *) 0);
	error_message ("  -n       notify user if any new newsgroups were created since last session", (char *) 0);
	error_message ("  -p file  print program with options [default=%s]", DEFAULT_PRINTER);
	error_message ("  -r       read news remotely from default NNTP server", (char *) 0);
	error_message ("  -R       read news saved by -S option (not yet implemented)", (char *) 0);
	error_message ("  -s dir   spool directory [default=%s]", SPOOLDIR);
	error_message ("  -S       save new news for later reading (batch mode)", (char *) 0);
	error_message ("  -u       update index files (batch mode)", (char *) 0);
	error_message ("  -U       update index files in the background while reading news", (char *) 0);
	error_message ("  -v       verbose output for -c -M -S -u and -Z options (batch mode)", (char *) 0);
	error_message ("  -z       start if any unread news", (char *) 0);
	error_message ("  -Z       return status indicating if any unread news (batch mode)", (char *) 0);
	error_message ("\nMail bug reports/comments to %s", BUG_REPORT_ADDRESS);
}

/*
 *  check/start if any new/unread articles
 */

int check_for_any_new_news (check_any_unread, start_any_unread)
	int check_any_unread;
	int start_any_unread;
{
	int i;
	
	if (check_any_unread) {
		i = check_start_save_any_news (CHECK_ANY_NEWS);
		exit (i);
	} else if (start_any_unread) {
		i = check_start_save_any_news (START_ANY_NEWS);
		if (i == -1) {		/* no new/unread news so exit */
			exit (0);
		}
		update = FALSE;
		return i;
	}
}

/*
 *  mail any new articles to specified user
 *  or
 *  save any new articles to savedir structure for later reading
 */

void save_or_mail_new_news ()
{
	int i;
	
	if (mail_news || save_news) {
		i = catchup;			/* set catchup to FALSE */
		catchup = FALSE;
		do_update ();
		catchup = i;			/* set catchup to previous value */
		if (mail_news) {
			check_start_save_any_news (MAIL_ANY_NEWS);
		} else {
			check_start_save_any_news (SAVE_ANY_NEWS);
		}
		tin_done (0);
	}
}

/*
 *  update index files
 */

void update_index_files ()
{
	if (update || update_fork) {
		COLS = DEFAULT_COLS;			/* set because curses has not started */ 
		if (update_fork) {
			catchup = FALSE;			/* turn off msgs when running forked */ 
			verbose = FALSE;
			if (fork () == 0) {			/* fork child to update indexes in background */
				nntp_startup ();		/* connect server if we are using nntp */
				thread_arts = FALSE;	/* stop threading to run faster */
				do_update ();
				nntp_finish ();			/* connect server if we are using nntp */
				exit (0);
			}
			update = FALSE;
		} else {
			thread_arts = FALSE;	/* stop threading to run faster */
			do_update ();
			exit (0);
		}
	}
}

/*
 *  display page of general info. for first time user.
 */

void show_intro_page ()
{
	if (cmd_line) {
		printf ("%s", cvers); 	
	} else {
		ClearScreen ();
		center_line (0, TRUE, cvers); 	
	}

	printf ("\n\nWelcome to tin, a full screen threaded Netnews reader. It can read news locally\n");
	printf ("(ie. <spool>/news) or remotely (-r option) from a NNTP  (Network News Transport\n");
	printf ("Protocol) server. tin -h lists the available command line options.\n\n");

	printf ("It has four newsreading levels,  the newsgroup selection page,  the group index\n");
	printf ("page, the thread listing page and the article viewer. Help is available at each\n");
	printf ("level by pressing the 'h' command.\n\n");

	printf ("Move up/down by using the terminal arrow keys or 'j' and 'k'.  Use PgUp/PgDn or\n");
	printf ("Ctrl-U and Ctrl-D to page up/down. Enter a newsgroup by pressing RETURN.\n\n");

	printf ("Articles, threads, tagged articles or articles matching a pattern can be mailed\n");
	printf ("('m' command), printed ('o' command), saved ('s' command), piped ('|' command).\n");
	printf ("Use the 'w' command  to post  a news  article,  the 'f'/'F' commands to  post a\n");
	printf ("follow-up  to  an existing  news article and the 'r'/'R' commands to  reply via\n");
	printf ("mail to an existing news articles author.  The 'M' command allows the operation\n");
	printf ("of tin to be configured via a menu.\n\n");

	printf ("For more in depth information please read  the supplied manual page and README.\n\n");
	printf ("Please send bug reports/comments to the programs author with the 'B' command.\n");
	fflush (stdout);

	if (! cmd_line) {
		continue_prompt ();
	}
}

