/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : misc.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 15-12-91
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

static char *mailbox_name = (char *) 0;
static int  mailbox_size;


void asfail(file, line, cond)
	char	*file;
	int	line;
	char	*cond;
{
	fprintf(stderr, "%s: assertion failure: %s (%d): %s\n",progname,file,line,cond);
	fflush (stderr);
	exit(1);
}


void copy_fp (fp_ip, fp_op, prefix)
	FILE *fp_ip;
	FILE *fp_op;
	char *prefix;
{
	extern int errno;
	char buf[8192];

	if (prefix == (char *) 0) {
		while (fgets (buf, sizeof (buf), fp_ip) != NULL) {
			if (fprintf (fp_op, "%s", buf) == EOF) {
				sprintf (msg, "Error: fprintf() failed in copy_fp(). errno=%d", errno);
				error_message (msg, (char *) 0);
			}
		}
	} else {
		while (fgets (buf, sizeof (buf), fp_ip) != NULL) {
			if (fprintf (fp_op, "%s%s", prefix, buf) == EOF) {
				sprintf (msg, "Error: fprintf() failed in copy_fp(). errno=%d", errno);
				error_message (msg, (char *) 0);
			}
		}
	}
}


char *get_val (env, def)
	char *env;		/* Environment variable we're looking for	*/
	char *def;		/* Default value if no environ value found	*/
{
	char *ptr;

	if ((ptr = (char *) getenv(env)) != NULL)
		return (ptr);
	else
		return (def);
}


int invoke_editor (nam)
	char *nam;
{
	char buf[LEN];
	char *my_editor;
	static char editor[LEN];
	static int first = TRUE;

	if (first) {
		my_editor = (char *) getenv ("VISUAL");

		strcpy (editor, my_editor != NULL ? my_editor : get_val ("EDITOR", DEFAULT_EDITOR));
		first = FALSE;
	}

#ifdef NO_START_LINE
	sprintf (buf, "%s %s", editor, nam);
#else
	sprintf (buf, "%s +%d %s", editor, start_line_offset, nam);
#endif

	printf ("%s", buf);
	return invoke_cmd (buf);
}


void shell_escape ()
{
	char shell[LEN];
	char *p;

#ifdef SIGTSTP
	SIGTYPE (*susp)();
#endif

	sprintf (msg, txt_shell_escape, default_shell_command);
	
	if (! prompt_string (msg, shell))
		strcpy (shell, get_val ("SHELL", DEFAULT_SHELL));

	for (p = shell; *p && (*p == ' ' || *p == '\t'); p++)
		continue;

	if (*p) {
		my_strncpy (default_shell_command, p, LEN);
	} else {
		if (default_shell_command[0]) {
			my_strncpy (shell, default_shell_command, LEN);
		} else {
			strcpy (shell, get_val ("SHELL", DEFAULT_SHELL));
		}
		p = shell;
	}

	ClearScreen ();
	sprintf (msg, "Shell Command (%s)", p);
	center_line (0, TRUE, msg);
	MoveCursor (INDEX_TOP, 0);
	
	EndWin ();
	Raw (FALSE);

	set_real_uid_gid ();

#ifdef SIGTSTP
	if (do_sigtstp)
		susp = signal (SIGTSTP, SIG_DFL);
#endif

	system (p);

#ifdef SIGTSTP
	if (do_sigtstp)
		signal (SIGTSTP, susp);
#endif

	set_tin_uid_gid ();

	Raw (TRUE);
	InitWin ();

	mail_setup ();

	continue_prompt ();

	if (draw_arrow_mark) {
		ClearScreen ();
	}
}


void tin_done (ret)
	int ret;
{
	char group_path[LEN], *p;
	int ask = TRUE;
	register int i, j;
	
	/*
	 * check if any groups were read & ask if they should marked read
	 */
	if (catchup_read_groups) {
		for (i = 0 ; i < group_top ; i++) {
			if (active[my_group[i]].read) {
				if (ask) {
					if (prompt_yn (LINES, "Catchup all groups entered during this session? (y/n): ", 'n')) {
						ask = FALSE;
						thread_arts = FALSE;	/* speeds up index loading */
					} else {
						break;
					}
				}
				sprintf (msg, "Catchup %s...", active[my_group[i]].name);
				wait_message (msg);
				strcpy (group_path, active[my_group[i]].name);
				for (p = group_path; *p; p++) {
					if (*p == '.') {
						*p = '/';
					}
				}
				index_group (active[my_group[i]].name, group_path);
				for (j = 0; j < top; j++) {
					arts[j].unread = ART_READ;
				}
				update_newsrc (active[my_group[i]].name, my_group[i], FALSE);
			}
		}
	}
	nntp_finish ();			/* disconnect from NNTP server */
	free_all_arrays ();		/* deallocate all arrays */
	ClearScreen ();
	EndWin ();
	Raw (FALSE);
	exit (ret);
}

/*
 *  Load the active file into active[] and create copy of active ~/.tin/active
 */

int read_active ()
{
	FILE *fp;
	char *p, *q, *r;
	char buf[LEN];
	char moderated;
	int created, i;
	long h;
	
	num_active = 0;

	if ((fp = open_active_fp ()) == NULL) {
		if (compiled_with_nntp) {
			printf ("Cannot open %s. Try %s -r to read news via NNTP.\n",
				active_file, progname);
		} else {
			printf (txt_cannot_open, active_file);
			putchar ('\n');
		}
		fflush (stdout);
		exit (1);
	}

	if (read_news_via_nntp && update == FALSE) {
		wait_message (txt_reading_active_file);
	}
	
	while (fgets (buf, sizeof (buf), fp) != NULL) {
		for (p = buf; *p && *p != ' '; p++)
			continue;
		if (*p != ' ') {
			error_message (txt_bad_active_file, buf);
			continue;
		}
		*p++ = '\0';

		if (num_active >= max_active)
			expand_active ();

		h = hash_groupname (buf);

		if (group_hash[h] == -1) {
			group_hash[h] = num_active;
		} else {				/* hash linked list chaining */
			for (i=group_hash[h]; active[i].next >= 0; i=active[i].next) {
				if (strcmp(active[i].name, buf) == 0) {
					goto read_active_continue;		/* kill dups */
				}
			}
			if (strcmp(active[i].name, buf) == 0)
				goto read_active_continue;
			active[i].next = num_active;
		}

		for (q = p; *q && *q != ' '; q++)
			continue;
		if (*q != ' ') {
			error_message (txt_bad_active_file, buf);
			continue;
		}
		*q++ = '\0';

		for (r = q; *r && *r != '\n'; r++) {
			if (*r == 'y' || *r == 'm') {
				moderated = *r;
				break;
			}
		}

		active[num_active].name = str_dup (buf);
		active[num_active].max = (long) atol (p);
		active[num_active].min = (long) atol (q);
		active[num_active].moderated = moderated;
		active[num_active].next = -1;			/* hash chaining */
		active[num_active].flag = UNSUBSCRIBED;	/* not in my_group[] yet */
		active[num_active].read = FALSE;		/* read/unread */
		active[num_active].thread = TRUE;		/* thread articles */

		num_active++;

read_active_continue:;

	}
	fclose (fp);

	/*
	 *  exit if active file is empty
	 */
	if (! num_active) {
		error_message ("%s contains no newsgroups. Exiting.", active_file);
		exit (1);
	}

	/*
	 *  create backup of LIBDIR/active for use by -n option to notify new groups 
	 */
	created = backup_active (TRUE);

	debug_print_active ();

	return (created);
}

/*
 *  Mark any groups in my_group[] that are in ~/.tin/unthread so they
 *  will not be threaded
 */
 
void mark_unthreaded_groups ()
{
	FILE *fp;
	char buf[LEN];
	int i;
	long h;

	set_real_uid_gid ();
	
	if ((fp = fopen (unthreadfile, "r")) == NULL) {
		error_message (txt_cannot_open, unthreadfile);
		set_tin_uid_gid ();
		return;
	}

	while (fgets (buf, sizeof (buf), fp) != NULL) {
		buf[strlen (buf)-1] = '\0'; 
		h = hash_groupname (buf);

		if (debug) {
			printf ("Unthreading %s...\n", buf);
		}

		if ((i = group_hash[h]) >= 0) {
			active[i].thread = FALSE;
		} else {
			for (i=group_hash[h]; active[i].next >= 0; i=active[i].next) {
				if (strcmp (active[i].name, buf) == 0) {
					active[i].thread = FALSE;
					break;
				}
			}
		}
	}
	fclose (fp);
	set_tin_uid_gid ();
}

/*
 *  create ~/.tin/active from LIBDIR/active if it does not exist 
 */

int backup_active (create)
	int create;
{
	char buf[LEN];
	FILE *fp;
	int created = FALSE;
	int i;
	struct stat sb;
	
	set_real_uid_gid ();
	
	sprintf (buf, "%s/active", rcdir);
	
	if (create) {
		if (stat (buf, &sb) != -1) {
			goto backup_active_done;
		}
	}
	
	if ((fp = fopen (buf, "w")) != NULL) {
		for (i = 0; i < num_active ; i++) {	/* for each group */
			fprintf (fp, "%s\n", active[i].name);
		}
		fclose (fp);
		chmod (buf, 0644);
		created = TRUE;
	}

backup_active_done:
	set_tin_uid_gid ();
	return (created);
}


#ifdef USE_MKDIR
mkdir (path, mode)
	char *path;
	int mode;
{
	char buf[LEN];
	struct stat sb;

	sprintf(buf, "mkdir %s", path);
	if (stat (path, &sb) == -1) {
		system (buf);
		chmod (path, mode);
	}
}
#endif


long hash_groupname (buf)		/* hash group name for fast lookup later */
	char *buf;
{
	unsigned long h;
	unsigned char *t = (unsigned char *) buf;

	h = *t++;
	while (*t)
		h = ((h << 1) ^ *t++) % TABLE_SIZE;

	return h;
}


void rename_file (old_filename, new_filename)
	char *old_filename;
	char *new_filename;
{	
	char buf[LEN];
	
	unlink (new_filename);
	if (link (old_filename, new_filename) == 1) {
		sprintf (buf, txt_rename_error, old_filename, new_filename);
		error_message (buf, "TWO");
		return;
	}
	if (unlink (old_filename) == -1) {
		sprintf (buf, txt_rename_error, old_filename, new_filename);
		error_message (buf, "THREE");
		return;
	}
}


char *str_dup (str)
	char *str;
{
	char *dup = (char *) 0;

	assert (str != (char *) 0);
	
	if (str) {
		dup = my_malloc (strlen (str)+1);
		strcpy (dup, str);
	}
	return dup;
}


int invoke_cmd (nam)
	char *nam;
{
	int ret;

#ifdef SIGTSTP
	SIGTYPE (*susp)();
#endif

	EndWin ();
	Raw (FALSE);
	set_real_uid_gid ();

#ifdef SIGTSTP
	if (do_sigtstp)
		susp = signal(SIGTSTP, SIG_DFL);
#endif

	ret = system (nam);

#ifdef SIGTSTP
	if (do_sigtstp)
		signal (SIGTSTP, susp);
#endif

	set_tin_uid_gid ();

	Raw (TRUE);
	InitWin ();

	return ret == 0;
}


void draw_percent_mark (cur_num, max_num)
	int cur_num;
	int max_num;
{
	char buf[32];
	int percent = 0;

	if (NOTESLINES <= 0) {
		return;
	}

	if (cur_num <= 0 && max_num <= 0) {
		return;
	}
		
	percent = cur_num * 100 / max_num;
	sprintf (buf, "%s(%d%%) [%d/%d]", txt_more, percent, cur_num, max_num);
	MoveCursor (LINES, (COLS - (int) strlen (buf))-(1+BLANK_PAGE_COLS));
	StartInverse ();	
	printf ("%s", buf);
	fflush (stdout);
	EndInverse ();
}


void set_real_uid_gid ()
{
#if defined(BSD) && ! defined(SINIX)
	if (setreuid (tin_uid, real_uid) == -1) {
		error_message ("Error setreuid(real) failed", (char *) 0);
	}
	if (setregid (tin_gid, real_gid) == -1) {
		error_message ("Error setregid(real) failed", (char *) 0);
	}
#else
	if (setuid (real_uid) == -1) {
		error_message ("Error setuid(real) failed", (char *) 0);
	}
	if (setgid (real_gid) == -1) {
		error_message ("Error setgid(real) failed", (char *) 0);
	}
#endif
}


void set_tin_uid_gid ()
{
#if defined(BSD) && ! defined(SINIX)
	if (setreuid (real_uid, tin_uid) == -1) {
		error_message ("Error setreuid(tin) failed", (char *) 0);
	}
	if (setregid (real_gid, tin_gid) == -1) {
		error_message ("Error setregid(tin) failed", (char *) 0);
	}
#else
	if (setuid (tin_uid) == -1) {
		error_message ("Error setuid(tin) failed", (char *) 0);
	}
	if (setgid (tin_gid) == -1) {
		error_message ("Error setgid(tin) failed", (char *) 0);
	}
#endif
}


void notify_groups ()
{
	char buf[LEN];
	FILE *fp;
	int group_not_found;
	int i, j, index;
	int num = 0;
	int update_old_active = FALSE;
	struct notify_t {
		char name[LEN];
		int len;
	} *old_active;
	
	set_real_uid_gid ();
	
	sprintf (buf, "%s/active", rcdir);
	
	if ((fp = fopen (buf, "r")) == NULL) {
		error_message (txt_cannot_open, buf);
		goto notify_groups_done;
	}

	Raw (FALSE);
	
	printf ("Checking active list for new newsgroups...");
	fflush (stdout);
	
	old_active = (struct notify_t *) my_malloc ((unsigned) sizeof (struct notify_t) * num_active);
	if (old_active == (struct notify_t *) 0) {
		error_message (txt_out_of_memory, progname);
		goto notify_groups_done;
	}
	
	while (fgets (old_active[num].name, sizeof (old_active[num].name), fp) != NULL) {
		old_active[num].len = strlen (old_active[num].name)-1;
		old_active[num].name[old_active[num].len] = '\0';
		num++;
		if (num == num_active) {
			break;
		}
	}

	for (i = 0 ; i < num_active ; i++) {	
		group_not_found = TRUE;
		for (j=0; j < num ; j++) {
			if (strncmp (old_active[j].name, active[i].name, old_active[j].len) == 0) {
				group_not_found = FALSE;		/* found it so read in next group */
				break;
			}
		}

		if (group_not_found) {
/*
&& strlen (active[i].name) == old_active[j].len) {
*/
			update_old_active = TRUE;
			printf ("\r\nSubscribe to %s (y/n): ", active[i].name);
			fflush (stdout);
			scanf ("%s", buf);
			if (buf[0] == 'y') {
				index = add_group (active[i].name, TRUE);
				subscribe (active[my_group[index]].name, ':',
					my_group[index], FALSE);
			}
			printf ("Checking...");
			fflush (stdout);
		}
	}
	fclose (fp);

	if (old_active != (struct notify_t *) 0) {
		free ((char *) old_active);
		old_active = (struct notify_t *) 0;
	}

	Raw (TRUE);

	/*
	 *  write active[] to ~/.tin/active
	 */
	if (update_old_active) {
		backup_active (FALSE);
	}

notify_groups_done:
	set_tin_uid_gid ();
}


void basename (dirname, program)
	char *dirname;		/* argv[0] */
	char *program;		/* progname is returned */
{
	int i;
	
	strcpy (program, dirname);
	
	for (i=(int) strlen (dirname)-1 ; i ; i--) {
		if (dirname[i] == '/') {
			strcpy (program, dirname+(i+1));
			break;
		}
	}
}


/*
 *  Record size of mailbox so we can detect if new mail has arrived
 */

void mail_setup ()
{
	struct stat buf;

	mailbox_name = get_val ("MAIL", mailbox);

	if (stat (mailbox_name, &buf) >= 0) {
		mailbox_size = buf.st_size;
	} else {
		mailbox_size = 0;
	}
}

/*
 *  Return TRUE if new mail has arrived
 */

int mail_check ()
{
	struct stat buf;

	if (mailbox_name != (char *) 0 &&
		stat(mailbox_name, &buf) >= 0 &&
		mailbox_size < buf.st_size) {
		return TRUE;
	}

	return FALSE;
}

/*
 *  Parse various From: lines into the component mail addresses and
 *  real names
 */

void parse_from (str, addr, name)
	char *str;
	char *addr;
	char *name;
{
	register int c;
	register char *cp, *ncp;
	int gotlt, lastsp, level;

	gotlt = 0;
	lastsp = 0;
	cp = addr;
	ncp = name;
	while (*str == ' ')
		++str;
	while (c = *str++)
		switch (c) {
		case '(':
			ncp = name;
			level = 1;
			while (*str != '\0' && level) {
				switch (c = *str++) {
				case '(':
					*ncp++ = c;
					level++;
					break;
				case ')':
					level--;
					if (level > 0)
						*ncp++ = c;
					break;
				default:
					*ncp++ = c;
					break;
				}
			}
			if (*str)
				str++;
			lastsp = 0;
			break;
		case ' ':
			if (str[0] == 'a' && str[1] == 't' && str[2] == ' ')
				str += 3, *cp++ = '@';
			else if (str[0] == '@' && str[1] == ' ')
				str += 2, *cp++ = '@';
			else
				lastsp = 1;
			if (ncp > name)
				*ncp++ = ' ';
			break;
		case '<':
			cp = addr;
			gotlt++;
			lastsp = 0;
			break;
		case '>':
			if (gotlt)
				goto done;
			/* FALL THROUGH CASE */
		default:
			if (lastsp) {
				lastsp = 0;
				*cp++ = ' ';
			}
			*cp++ = c;
			if (! gotlt)
				*ncp++ = c;
			break;
		}
done:
	*cp = 0;
	while (ncp>name && ncp[-1]==' ')
		--ncp;
	*ncp = 0;
	if (*addr == '@') {
		char buf [512];

		strcpy (buf, addr);
		strcpy (addr, "root");
		strcat (addr, buf);
	}
}

/*
 *  Convert a string to a long, only look at first n characters
 */

long my_atol (s, n)
	char *s;
	int n;
{
	long ret = 0;

	while (*s && n--) {
		if (*s >= '0' && *s <= '9')
			ret = ret * 10 + (*s - '0');
		else
			return -1;
		s++;
	}

	return ret;
}

/*
 *  Return a pointer into s eliminating any leading Re:'s.  Example:
 *
 *	  Re: Reorganization of misc.jobs
 *	  ^   ^
 */

char *eat_re (s)
	char *s;
{

	while (*s == 'r' || *s == 'R') {
		if ((*(s+1) == 'e' || *(s+1) == 'E')) {
			if (*(s+2) == ':')
				s += 3;
			else if (*(s+2) == '^' && isdigit(*(s+3)) && *(s+4) == ':')
				s += 5;			/* hurray nn */
			else
				break;
		} else
			break;
		while (*s == ' ')
			s++;
	}

	return s;
}

/*
 *  Hash the subjects (after eating the Re's off) for a quicker
 *  thread search later.  We store the hashes for subjects in the
 *  index file for speed.
 */

long hash_s (s)
	char *s;
{
	long h = 0;
	unsigned char *t = (unsigned char *) s;

	while (*t)
		h = h * 64 + *t++;

	return h;
}

/*
 *  strncpy that stops at a newline and null terminates
 */

void my_strncpy(p, q, n)
	char *p;
	char *q;
	int n;
{
	while (n--) {
		if (! *q || *q == '\n')
			break;
		*p++ = *q++;
	}
	*p = '\0';
}


int untag_all_articles ()
{
	int untagged = FALSE;
	register int i;

	for (i=0 ; i < top ; i++) {
		if (arts[i].tagged) {
			arts[i].tagged = FALSE;
			untagged = TRUE;
		}
	}
	num_of_tagged_files = 0;

	return (untagged);
}


/*
 * ANSI C strstr () - Uses Boyer-Moore algorithm. Downloaded from net.
 */
 
char *str_str (text, pattern, patlen)
	char *text;
	char *pattern;
	int patlen;
{
	register unsigned char *p, *t;
	register int i, p1, j, *delta;
	int deltaspace[256];
	int textlen;

	textlen = strlen (text);

	/* algorithm fails if pattern is empty */
	if ((p1 = patlen) == 0)
		return (text);

	/* code below fails (whenever i is unsigned) if pattern too long */
	if (p1 > textlen)
		return (NULL);

	/* set up deltas */
	delta = deltaspace;
	for (i = 0; i <= 255; i++)
		delta[i] = p1;
	for (p = (unsigned char *) pattern, i = p1; --i > 0;)
		delta[*p++] = i;

	/*
	 * From now on, we want patlen - 1.
	 * In the loop below, p points to the end of the pattern,
	 * t points to the end of the text to be tested against the
	 * pattern, and i counts the amount of text remaining, not
	 * including the part to be tested.
	 */
	p1--;
	p = (unsigned char *) pattern + p1;
	t = (unsigned char *) text + p1;
	i = textlen - patlen;
	for (;;) {
		if (*p == *t && memcmp((p - p1), (t - p1), p1) == 0)
			return ((char *)t - p1);
		j = delta[*t];
		if (i < j)
			break;
		i -= j;
		t += j;
	}
	return (NULL);
}


void get_author (thread, respnum, str)
	int thread;
	int respnum;
	char *str;
{	
	extern int threaded_on_subject;
	int author;
	int len_from = max_from;
	
	if (thread) {
		if (threaded_on_subject) {
			len_from = max_subj+max_from;
		} else {
			len_from = max_from;
		}
		author = SHOW_FROM_BOTH;
	} else {
		author = show_author;
	}

	switch (author) {
		case SHOW_FROM_ADDR:
			my_strncpy (str, arts[respnum].from, len_from);
			break;
		case SHOW_FROM_NAME:
			my_strncpy (str, arts[respnum].name, len_from);
			break;
		case SHOW_FROM_BOTH:
			if (arts[respnum].name != arts[respnum].from) { 
				sprintf (msg, "%s (%s)", arts[respnum].name, arts[respnum].from);
				my_strncpy (str, msg, len_from);
			} else { 
				my_strncpy (str, arts[respnum].from, len_from);
			}
			break;
	}
}


void toggle_inverse_video ()
{
	inverse_okay = !inverse_okay;
	if (inverse_okay) {
#ifndef USE_INVERSE_HACK	
		draw_arrow_mark = FALSE;
#endif		
		info_message (txt_inverse_on);
	} else {
		draw_arrow_mark = TRUE;
		info_message (txt_inverse_off);
	}
}


int get_arrow_key ()
{
	int ch;
	
	ch = ReadCh ();
	if (ch == '[' || ch == 'O')
		ch = ReadCh();
	switch (ch) {
		case 'A':
		case 'D':
		case 'i':
			return KEYMAP_UP;

		case 'B':
		case 'C':
			return KEYMAP_DOWN;

		case 'I':		/* ansi  PgUp */
		case 'V':		/* at386 PgUp */
		case 'S':		/* 97801 PgUp */
		case 'v':		/* emacs style */
			return KEYMAP_PAGE_UP;

		case 'G':		/* ansi  PgDn */
		case 'U':		/* at386 PgDn */
		case 'T':		/* 97801 PgDn */
			return KEYMAP_PAGE_DOWN;

		case 'H':		/* at386  Home */
			return KEYMAP_HOME;
					
		case 'F':		/* ansi  End */
		case 'Y':		/* at386  End */
			return KEYMAP_END;

		default:
			return KEYMAP_UNKNOWN;
	}
}
