/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : open.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 08-02-92
 *  Notes     : reads news locally (ie. /usr/spool/news) or via NNTP
 *  Copyright : (c) Copyright 1991-92 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"
#ifdef USE_NNTP
#	include	"nntp.h"
#endif

/* Hopefully one of these is right for you. */

#ifdef BSD
#	ifdef SINIX
#		include <dir.h>
#	else
#		include <sys/dir.h>
#	endif
#	define		DIR_BUF		struct direct
#	define		D_LENGTH	d_namlen
#endif
#ifdef M_XENIX
#	include <sys/ndir.h>
#	define		DIR_BUF		struct direct
#	define		D_LENGTH	d_namlen
#endif
#ifndef DIR_BUF
#	ifdef sorix960
#		include	<sys/dirent.h>
#	else
#		include	<dirent.h>
#	endif
#	define		DIR_BUF		struct dirent
#	define		D_LENGTH	d_reclen
#endif

#ifdef USE_NNTP
int compiled_with_nntp = TRUE;		/* used in mail_bug_report() info */
#else
int compiled_with_nntp = FALSE;
#endif

#ifdef NO_POSTING
int	can_post = FALSE;
#else
int	can_post = TRUE;
#endif

char server_name[LEN];


char *is_remote ()
{
	server_name[0] = '\0';
	
#ifdef USE_NNTP
	if (read_news_via_nntp) {
		if (nntp_server[0]) {
			sprintf (server_name, " (%s)", nntp_server);
		} else {
			if (getserverbyfile (NNTP_SERVER_FILE)) {
				sprintf (server_name, " (%s)", getserverbyfile (NNTP_SERVER_FILE));
			} else {
				strcpy (server_name, " (NO SERVER)");
			}
		}
	}
#endif

	return (server_name);
}


void nntp_startup ()
{
#ifdef USE_NNTP	
	char *server;
	int ret;
	extern char *getenv();

	if (read_news_via_nntp) {
		if (nntp_server[0]) {
			server = nntp_server;
		} else {
			server = getserverbyfile (NNTP_SERVER_FILE);
		}
		if (server == (char *) 0) {
			fprintf (stderr, txt_cannot_get_nntp_server_name);
			fprintf (stderr, txt_server_name_in_file_env_var, NNTP_SERVER_FILE);
			exit(1);
		}

		if (update == FALSE) {
			printf (txt_connecting, server);
			fflush (stdout);
		}
		
		ret = server_init (server);
		if (update == FALSE) {
			putchar ('\n');
		}
/*
		handle_server_response (ret, server);
*/		
		switch (ret) {
		case OK_CANPOST:
#ifndef NO_POSTING		
			can_post = TRUE;
#endif			
			break;

		case OK_NOPOST:
			can_post = FALSE;
			break;	

		case -1:
			fprintf (stderr, txt_failed_to_connect_to_server, server);
			exit (1);

		default:
			fprintf (stderr, txt_rejected_by_nntpserver, ret);
			exit (1);
		}
	}
#endif	
}


void nntp_finish ()
{
#ifdef USE_NNTP
	if (read_news_via_nntp) {
		close_server ();
	}
#endif	
}


FILE *open_active_fp ()
{
	if (read_news_via_nntp) {
#ifdef USE_NNTP
		put_server ("list");
		if (get_respcode () != OK_GROUPS) {
			return NULL;
		}
		return nntp_to_fp ();
#else
		return NULL;
#endif		
	} else {
		return fopen (active_file, "r");
	}
}


FILE *open_art_fp (group_path, art)
	char *group_path;
	long art;
{
	char buf[LEN];
	int respcode;
	struct stat sb;
	extern long note_size;

	if (read_news_via_nntp) {
#ifdef USE_NNTP
		sprintf (buf, "article %ld", art);

		put_server (buf);
		if ((respcode = get_respcode ()) != OK_ARTICLE) {
			sprintf (msg, "Cannot open art on NNTP server. Respcode %d", respcode);
			error_message (msg);
			return (FILE *) 0;
		}

		return nntp_to_fp ();
#else
		return (FILE *) 0;
#endif
	} else {
		sprintf (buf, "%s/%s/%ld", SPOOLDIR, group_path, art);

		if (stat (buf, &sb) < 0) {
			note_size = 0;
		} else {
			note_size = sb.st_size;
		}
		return fopen (buf, "r");
	}
}


int open_header_fd (group_path, art)
	char *group_path;
	long art;
{
	char buf[LEN];

	if (read_news_via_nntp) {
#ifdef USE_NNTP	
		sprintf(buf, "head %ld", art);
		
		put_server (buf);
		if (get_respcode () != OK_HEAD) {
			return -1;
		}

		return nntp_to_fd ();
#else
		return -1;
#endif		
	} else {
		sprintf (buf, "%s/%s/%ld", SPOOLDIR, group_path, art);
		return open (buf, 0);
	}
}

/*
 *  Longword comparison routine for the qsort()
 */

int base_comp (p1, p2)
	char *p1;
	char *p2;
{
	long *a = (long *) p1;
	long *b = (long *) p2;

	if (*a < *b)
		return -1;
	if (*a > *b)
		return 1;
	return 0;
}


/*
 *  Read the article numbers existing in a group's spool directory
 *  into base[] and sort them.  top_base is one past top.
 */

void setup_base (group, group_path)
	char *group;
	char *group_path;
{
	char buf[LEN];
#ifdef USE_NNTP
	char line[NNTP_STRLEN];
#endif
	DIR *d;
	DIR_BUF *e;
	long art, start, last, dummy, count;

	top_base = 0;

	if (read_news_via_nntp) {
#ifdef USE_NNTP
		sprintf (buf, "group %s", group);
		put_server (buf);

		if (get_server (line, NNTP_STRLEN) == -1) {
			fprintf (stderr, txt_connection_to_server_broken);
			tin_done (1);
		}

		if (atoi(line) != OK_GROUP) {
			return;
		}

		sscanf (line,"%ld %ld %ld %ld", &dummy, &count, &start, &last);
		if (last - count > start) {
			start = last - count;
		}

		while (start <= last) {
			if (top_base >= max_art) {
				expand_art();
			}
			base[top_base++] = start++;
		}
#else
		return; 
#endif
	} else {
		sprintf (buf, "%s/%s", SPOOLDIR, group_path);

		if (access (buf, 4) != 0) {
			return;
		}

		d = opendir (buf);
		if (d != NULL) {
			while ((e = readdir (d)) != NULL) {
				art = my_atol (e->d_name, (int) e->D_LENGTH);
				if (art >= 0) {
					if (top_base >= max_art)
						expand_art ();
					base[top_base++] = art;
				}
			}
			closedir (d);
			qsort ((char *) base, top_base, sizeof (long), base_comp);
		}
	}
}

/*
 *  get_respcode
 *  get a response code from the server and return it to the caller
 */

int get_respcode ()
{
#ifdef USE_NNTP
	char line[NNTP_STRLEN];

	if (get_server (line, NNTP_STRLEN) == -1) {
		fprintf (stderr, txt_connection_to_server_broken);
		tin_done (1);
	}

	return atoi (line);
#endif
}


int stuff_nntp (fnam)
	char *fnam;
{
#ifdef USE_NNTP
	FILE *fp;
	char line[NNTP_STRLEN];
	extern char *mktemp();
	struct stat sb;
	extern long note_size;

	strcpy(fnam, "/tmp/tin_nntpXXXXXX");
	mktemp(fnam);

	if ((fp = fopen(fnam, "w")) == NULL) {
		error_message (txt_stuff_nntp_cannot_open, fnam);
		return FALSE;
	}

	while (1) {
		if (get_server(line, NNTP_STRLEN) == -1) {
			fprintf(stderr, txt_connection_to_server_broken);
			tin_done (1);
		}
		if (strcmp(line, ".") == 0)
			break;			/* end of text */
		strcat(line, "\n");
		if (line[0] == '.')		/* reduce leading .'s */
			fputs(&line[1], fp);
		else
			fputs(line, fp);
	}
	fclose(fp);

	if (stat(fnam, &sb) < 0)
		note_size = 0;
	else
		note_size = sb.st_size;

	return TRUE;
#else
	return TRUE;
#endif
}


FILE *nntp_to_fp ()
{
#ifdef USE_NNTP
	char fnam[LEN];
	static FILE *fp = NULL;

	if (fp != NULL) {
		fclose (fp);
	}
	
	if (! stuff_nntp (fnam)) {
		return NULL;
	}

	if ((fp = fopen (fnam, "r")) == NULL) {
		error_message (txt_nntp_to_fp_cannot_reopen, fnam);
		return NULL;
	}
	
	unlink (fnam);
	return fp;
#else
	return NULL;
#endif
}


int nntp_to_fd ()
{
#ifdef USE_NNTP
	char fnam[LEN];
	static int fd = -1;

	if (fd != -1) {
		close (fd);
	}
	
	if (! stuff_nntp (fnam)) {
		return -1;
	}

	if ((fd = open (fnam, 0)) == -1) {
		error_message (txt_nntp_to_fd_cannot_reopen, fnam);
		return -1;
	}
	
	unlink (fnam);
	return fd;
#else
	return -1;
#endif
}
