/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : prompt.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 25-11-91
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Rich Skrenta & Iain Lea
 *              You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

/*
 *  prompt_num
 *  get a number from the user
 *  Return -1 if missing or bad number typed
 */

int prompt_num (ch, prompt)
	char ch;
	char *prompt;
{
	char *p;
	int num;

	clear_message ();

	sprintf (msg, "%c", ch);

	if ((p = getline (prompt, TRUE, msg)) != (char *) 0) {
		strcpy (msg, p);
		num = atoi (msg);
	} else {
		num = -1;
	}

	clear_message ();
	return (num);
}

/*
 *  prompt_string
 *  get a string from the user
 *  Return TRUE if a valid string was typed, FALSE otherwise
 */

int prompt_string (prompt, buf)
	char *prompt;
	char *buf;
{
	char *p;

	clear_message ();

	if ((p = getline (prompt, FALSE, (char *) 0)) == (char *) 0) {
		buf[0] = '\0';
		clear_message ();
		return FALSE;
	}
	strcpy (buf, p);
	
	clear_message ();

	return TRUE;
}

/*
 *  prompt_menu_string
 *  get a string from the user
 *  Return TRUE if a valid string was typed, FALSE otherwise
 */

int prompt_menu_string (line, col, var)
	int line;
	int col;
	char *var;
{
	char *p;

	MoveCursor (line, col);

	if ((p = getline ((char *) 0, FALSE, var)) == (char *) 0) {
		return FALSE;
	}
	strcpy (var, p);
	
	return TRUE;

#ifdef XXX
	char buf[LEN];
	char ch;
	int i, len;
	int max_len;

	MoveCursor (line, col);
	buf[0] = '\0';
	len = 0;
	max_len = (COLS - col)-2;
	ch = (char) ReadCh();

	while (ch != '\n' && ch != '\r') {
		if (ch == ESC) {		/* Esc pressed so abort */
			return FALSE;
		}
		if (ch == 8 || ch == 127) {
			if (len) {
				len--;
				buf[len] = '\0';
				putchar('\b');
				putchar(' ');
				putchar('\b');
			} else {
				strcpy (var, buf);
				MoveCursor(line, col);
				CleartoEOLN();
			}
		} else if (ch == 21) {	/* control-U	*/
			for (i = len; i > 0; i--) {
				putchar('\b');
				putchar(' ');
				putchar('\b');
			}
			buf[0] = '\0';
			len = 0;
		} else if (ch >= ' ' && len < max_len) {
			buf[len++] = ch;
			buf[len] = '\0';
			putchar (ch);
		} else
			putchar(7);
		fflush(stdout);
		ch = (char) ReadCh();
		if(ch == EOF)
			return FALSE;
		ch &= 0xFF;
	}

	if (buf[0]) {
		strcpy (var, buf);
	}
	return TRUE;
#endif
}


int prompt_yn (line, prompt, default_ch)
	int line;
	char *prompt;
	char default_ch;
{
	char ch;

	MoveCursor (line, 0);
	CleartoEOLN ();
	printf ("%s%c", prompt, default_ch);
	fflush (stdout);
	MoveCursor (line, (int) strlen (prompt));

	if ((ch = (char) ReadCh()) == CR) {
		ch = default_ch;
	}	

	if (line == LINES) {
		clear_message();
	} else {
		MoveCursor (line, (int) strlen (prompt));
		if (ch == ESC) {
			printf ("%c", default_ch);
		} else {
			printf ("%c", ch);
		}
		fflush (stdout);
	}

	return (ch == 'y' ? TRUE : FALSE);
}


void prompt_on_off (row, col, var, help_text, prompt_text)
	int row;
	int col;
	int *var;
	char *help_text;
	char *prompt_text;
{
	int ch, var_orig;

	var_orig = *var;

	show_menu_help (help_text);
	do {
		MoveCursor (row, col + (int) strlen (prompt_text));
		if ((ch = (char) ReadCh ()) == ' ') {
			*var = !*var;
			printf ("%s", (*var ? "ON " : "OFF"));
			fflush (stdout);
		}
	} while (ch != CR && ch != ESC);

	if (ch == ESC) {
		*var = var_orig;
		printf ("%s", (*var ? "ON " : "OFF"));
		fflush (stdout);
	}
}


void continue_prompt ()
{
	char ch;
	
	info_message (txt_hit_any_key);
	ch = (char) ReadCh ();
}


