/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : rcfile.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 23-01-92
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char index_file[LEN];
extern int index_point;

static int COL1;
static int COL2;
static int COL3;

/*
 *  read_rcfile - read defaults from ~/.tin/tinrc
 */

int read_rcfile ()
{
	char buf[LEN];
	FILE *fp;

	if ((fp = fopen (rcfile, "r")) != NULL) {
		while (fgets (buf, sizeof buf, fp) != NULL) {
			if (buf[0] != '#') { 
				if (strncmp (buf, "save_archive=", 13) == 0) {
					save_archive_name = (strncmp (&buf[13], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "save_separate=", 14) == 0) {
					save_separate = (strncmp (&buf[14], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "mark_saved_read=", 16) == 0) {
					mark_saved_read = (strncmp (&buf[16], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "kill_articles=", 14) == 0) {
					kill_articles = (strncmp (&buf[14], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "draw_arrow=", 11) == 0) {
					draw_arrow_mark = (strncmp (&buf[11], "ON", 2) == 0 ? TRUE : FALSE);
					if (draw_arrow_mark == FALSE && inverse_okay == FALSE) {
						inverse_okay = TRUE;
					}
				} else if (strncmp (buf, "print_header=", 13) == 0) {
					print_header = (strncmp (&buf[13], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "pos_first_unread=", 17) == 0) {
					pos_first_unread = (strncmp (&buf[17], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "full_page_scroll=", 17) == 0) {
					full_page_scroll = (strncmp (&buf[17], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "catchup_read_groups=", 20) == 0) {
					catchup_read_groups = (strncmp (&buf[20], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "thread_articles=", 16) == 0) {
					thread_arts = (strncmp (&buf[16], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "show_only_unread=", 17) == 0) {
					show_only_unread = (strncmp (&buf[17], "ON", 2) == 0 ? TRUE : FALSE);
				} else if (strncmp (buf, "show_author=", 12) == 0) {
					default_show_author = atoi (&buf[12]);
				} else if (strncmp (buf, "post_process_type=", 18) == 0) {
					post_proc_type = atoi (&buf[18]);
					switch (post_proc_type) {
						case POST_PROC_NONE:
							proc_ch_default = 'n';
							break;
						case POST_PROC_SHAR:
							proc_ch_default = 's';
							break;
						case POST_PROC_UUDECODE:
							proc_ch_default = 'u';
							break;
						case POST_PROC_UUD_LST_ZOO:
							proc_ch_default = 'U';
							break;
						case POST_PROC_UUD_EXT_ZOO:
							proc_ch_default = 'U';
							break;
					}
				} else if (strncmp (buf, "sort_article_type=", 18) == 0) {
					sort_art_type = atoi (&buf[18]);
				} else if (strncmp (buf, "savedir=", 8) == 0) {
					strncpy (savedir, &buf[8], LEN);
					savedir[strlen (savedir) - 1] = '\0';
					if (savedir[0] == '.' && strlen (savedir) == 1) {
#if defined(BSD) && ! defined(SINIX)
						getwd (buf);	
#else
						getcwd (buf, LEN);
#endif
						my_strncpy (savedir, buf, LEN);
					}
				} else if (strncmp (buf, "maildir=", 8) == 0) {
					strncpy (maildir, &buf[8], LEN);
					maildir[strlen (maildir) - 1] = '\0';
				} else if (strncmp (buf, "printer=", 8) == 0) {
					strncpy (printer, &buf[8], LEN);
					printer[strlen (printer) - 1] = '\0';
				} else if (strncmp (buf, "spooldir=", 9) == 0) {
					strncpy (spooldir, &buf[9], LEN);
					spooldir[strlen (spooldir) - 1] = '\0';
				} else if (strncmp (buf, "signature=", 10) == 0) {
					strncpy (signature, &buf[10], LEN);
					signature[strlen (signature) - 1] = '\0';
				} else if (strncmp (buf, "sig=", 4) == 0) {
					strncpy (sig, &buf[4], LEN);
					sig[strlen (sig) - 1] = '\0';
				}
			}
		}
		fclose (fp);
		return TRUE;		
	}
	return FALSE;		
}

/*
 *  write_rcfile - write defaults to ~/.tin/tinrc
 */

void write_rcfile ()
{
	FILE *fp;

	set_real_uid_gid ();
	
	if ((fp = fopen (rcfile, "w")) != NULL) {
		wait_message (txt_saving);

		fprintf (fp, "# if ON articles/threads with Archive-name: in mail header will\n");
		fprintf (fp, "# be automatically saved with the Archive-name & part/patch no.\n");
		fprintf (fp, "save_archive=%s\n\n", (save_archive_name ? "ON" : "OFF"));
		fprintf (fp, "# if ON articles of a threads will be saved to separate files\n");
		fprintf (fp, "# otherwise the whole thread will be saved to one file\n");
		fprintf (fp, "save_separate=%s\n\n", (save_separate ? "ON" : "OFF"));
		fprintf (fp, "# if ON mark articles that are saved as read\n");
		fprintf (fp, "mark_saved_read=%s\n\n", (mark_saved_read ? "ON" : "OFF"));
		fprintf (fp, "# if ON use -> otherwise highlighted bar for selection\n");
		fprintf (fp, "draw_arrow=%s\n\n", (draw_arrow_mark ? "ON" : "OFF"));
		fprintf (fp, "# if ON kill articles that match kill file\n");
		fprintf (fp, "kill_articles=%s\n\n", (kill_articles ? "ON" : "OFF"));
		fprintf (fp, "# if ON print all of mail header otherwise Subject: & From: lines\n");
		fprintf (fp, "print_header=%s\n\n", (print_header ? "ON" : "OFF"));
		fprintf (fp, "# if ON put cursor at first unread art in group otherwise last art\n");
		fprintf (fp, "pos_first_unread=%s\n\n", (pos_first_unread ? "ON" : "OFF"));
		fprintf (fp, "# if ON scroll full page of groups/articles otherwise half a page\n");
		fprintf (fp, "full_page_scroll=%s\n\n", (full_page_scroll ? "ON" : "OFF"));
		fprintf (fp, "# if ON ask user if read groups should all be marked read\n");
		fprintf (fp, "catchup_read_groups=%s\n\n", (catchup_read_groups ? "ON" : "OFF"));
		fprintf (fp, "# part of from field to display 0) none 1) address 2) full name 3) both\n");
		fprintf (fp, "show_author=%d\n\n", default_show_author);
		fprintf (fp, "# type of post processing to perform after saving articles.\n");
		fprintf (fp, "# 0) none 1) shar 2) uudecode 3) uud & list zoo 4) uud & extract zoo 5) patch.\n");
		fprintf (fp, "post_process_type=%d\n\n", post_proc_type);
		fprintf (fp, "# if ON and group not in ~/.tin/unthread articles will be threaded.\n");
		fprintf (fp, "thread_articles=%s\n\n", (thread_arts ? "ON" : "OFF"));
		fprintf (fp, "# if ON show only new/unread articles otherwise show all.\n");
		fprintf (fp, "show_only_unread=%s\n\n", (show_only_unread ? "ON" : "OFF"));
		fprintf (fp, "# sort articles by 0) nothing 1) Subject (descending) 2) Subject (ascending)\n");
		fprintf (fp, "# 3) From (descend) 4) From (ascend) 5) Date (descend) 6) Date (ascend)\n");
		fprintf (fp, "sort_article_type=%d\n\n", sort_art_type);
		fprintf (fp, "# (-d) directory where articles/threads are saved\n");
		fprintf (fp, "savedir=%s\n\n", savedir);
		fprintf (fp, "# (-M) directory where articles/threads are saved in mailbox format\n");	
		fprintf (fp, "maildir=%s\n\n", maildir);	
		fprintf (fp, "# (-p) print program with parameters used to print articles/threads\n");
		fprintf (fp, "printer=%s\n\n", printer);
		fprintf (fp, "# (-s) directory where news is spooled\n");
		fprintf (fp, "spooldir=%s\n\n", spooldir);

		fprintf (fp, "# .signature file used for replies, followups\n");
		fprintf (fp, "signature=%s\n\n", signature);
		fprintf (fp, "# .Sig file used for postings\n");
		fprintf (fp, "sig=%s\n\n", sig);

		fclose (fp);
		chmod (rcfile, 0600);
	}
	set_tin_uid_gid ();
}

/*
 *  options menu so that the user can dynamically change parameters
 */
 
int change_rcfile (group, kill_at_once)
	char *group;
	int kill_at_once;
{
	char *str;
	int ch, i;
	int kill_changed = FALSE;
	int orig_kill_state;
	int orig_show_only_unread;
	int orig_thread_arts;
	int option;
	int ret_code = NO_KILLING;
	int var_orig;
	
#ifdef SIGTSTP
	SIGTYPE (*susp)();

	if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
		sigemptyset (&rcfile_act.sa_mask);
		rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
		rcfile_act.sa_handler = SIG_DFL;
		sigaction (SIGTSTP, &rcfile_act, &old_act);
#else
		susp = signal (SIGTSTP, SIG_DFL);
#endif
	}
#endif

	COL1 = 0;
	COL2 = ((COLS / 3) * 1) + 1;
	COL3 = ((COLS / 3) * 2) + 2;

	show_rcfile_menu ();

	while (1) {

#ifdef SIGTSTP
		if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
			sigemptyset (&rcfile_act.sa_mask);
			rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
			rcfile_act.sa_handler = rcfile_suspend;
			sigaction (SIGTSTP, &rcfile_act, 0L);
#else
			signal (SIGTSTP, rcfile_suspend);
#endif
		}
#endif
		MoveCursor (LINES, 0);
		ch = ReadCh ();
		if (ch >= '1' && ch <= '9') {
			option = prompt_num (ch, "Enter option number> ");
		} else {
			if (ch == 'q' || ch == ESC) {
				option = -1;
			} else {
				option = 0;
			}
		}
#ifdef SIGTSTP
		if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
			sigemptyset (&rcfile_act.sa_mask);
			rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
			rcfile_act.sa_handler = SIG_IGN;
			sigaction (SIGTSTP, &rcfile_act, 0L);
#else
			signal (SIGTSTP, SIG_IGN);
#endif
		}
#endif
		switch (option) {
			case -1:
				clear_note_area ();
				return ret_code;
				/*NOTREACHED*/				
			case 0:
				write_rcfile ();
				if (kill_changed) {
					if (kill_at_once) {
						if (kill_articles) {
							read_kill_file ();
							if (kill_any_articles (group)) {
								reload_index_file (group, TRUE);	/* kill arts */
							}
						} else {
							reload_index_file (group, FALSE);	/* add killed arts */
						}
					}
					ret_code = KILLING;
				}
				clear_note_area ();
#ifdef SIGTSTP
				if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
					sigemptyset (&rcfile_act.sa_mask);
					rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
					rcfile_act.sa_handler = SIG_IGN;
					sigaction (SIGTSTP, &old_act, 0L);
#else
					signal (SIGTSTP, susp);
#endif
				}
#endif
				return ret_code;
			
			case 1:		/* auto save */
				prompt_on_off (INDEX_TOP, COL1, &save_archive_name, 
					txt_help_autosave, txt_opt_autosave);
				break;

			case 2:		/* save sperate */
				prompt_on_off (INDEX_TOP, COL2, &save_separate, 
					txt_help_save_separate, txt_opt_save_separate);
				break;
			
			case 3:		/* mark saved articles read */
				prompt_on_off (INDEX_TOP, COL3, &mark_saved_read, 
					txt_help_mark_saved_read, txt_opt_mark_saved_read);
				break;

			case 4:		/* kill articles */
				orig_kill_state = kill_articles;
				show_menu_help (txt_help_kill_articles);
				do {
					MoveCursor (INDEX_TOP+2, COL1 + (int) strlen (txt_opt_kill_articles));
					if ((ch = ReadCh()) == ' ') {
						kill_articles = !kill_articles;
						kill_changed = (kill_articles != orig_kill_state ? TRUE : FALSE);
						printf ("%s", (kill_articles ? "ON " : "OFF"));
						fflush (stdout);
					}
				} while (ch != CR && ch != ESC);
				break;

			case 5:		/* draw -> / highlighted bar */
				prompt_on_off (INDEX_TOP+2, COL2, &draw_arrow_mark, 
					txt_help_draw_arrow, txt_opt_draw_arrow);
				if (draw_arrow_mark == FALSE && inverse_okay == FALSE) {
					inverse_okay = TRUE;
				}
				break;

			case 6:		/* print header */
				prompt_on_off (INDEX_TOP+2, COL3, &print_header, 
					txt_help_print_header, txt_opt_print_header);
				break;
			
			case 7:		/* position cursor at first / last unread art */
				prompt_on_off (INDEX_TOP+4, COL1, &pos_first_unread, 
					txt_help_pos_first_unread, txt_opt_pos_first_unread);
				break;

			case 8:		/* scroll half/full page of groups/articles */
				prompt_on_off (INDEX_TOP+4, COL2, &full_page_scroll, 
					txt_help_page_scroll, txt_opt_page_scroll);
				break;

			case 9:		/* catchup read groups when quitting */
				prompt_on_off (INDEX_TOP+4, COL3, &catchup_read_groups, 
					txt_help_catchup_groups, txt_opt_catchup_groups);
				break;

			case 10:	/* thread/unthread all groups except those in ~/.tin/unthreaded */
				orig_thread_arts = thread_arts;	
				prompt_on_off (INDEX_TOP+6, COL1, &thread_arts, 
					txt_help_thread_arts, txt_opt_thread_arts);
				if (thread_arts != orig_thread_arts || group != (char *) 0) {
					make_threads (TRUE);
					find_base (show_only_unread);
				}
				break;

			case 11:	/* show all arts or just new/unread arts */
				orig_show_only_unread = show_only_unread;	
				prompt_on_off (INDEX_TOP+6, COL2, &show_only_unread, 
					txt_help_show_only_unread, txt_opt_show_only_unread);
				if (show_only_unread != orig_show_only_unread || group != (char *) 0) {
					make_threads (TRUE);
					find_base (show_only_unread);
					if (space_mode) {
						for (i = 0; i < top_base; i++) {
							if (new_responses (i)) {
								break;
							}
						}
						if (i < top_base) {
							index_point = i;
						} else {
							index_point = top_base - 1;
						}
					} else {
						index_point = top_base - 1;
					}
				}
				break;

			case 13:		/* show subject & author / subject only */
				var_orig = show_author;
				show_menu_help (txt_help_show_author);
				do {
					MoveCursor (INDEX_TOP+8, COL1 + (int) strlen (txt_opt_show_author));
					if ((ch	= ReadCh()) == ' ') {
						if (show_author + 1 > SHOW_FROM_BOTH) {
							show_author = SHOW_FROM_NONE;
						} else {
							show_author++;
						}
						switch (show_author) {
							case SHOW_FROM_NONE:
								str = txt_show_from_none;
								break;
							case SHOW_FROM_ADDR:
								str = txt_show_from_addr;
								break;
							case SHOW_FROM_NAME:
								str = txt_show_from_name;
								break;
							case SHOW_FROM_BOTH:
								str = txt_show_from_both;
								break;
						}
						printf ("%s", str);
						fflush (stdout);
					}
				} while (ch != CR && ch != ESC);

				if (ch == ESC) {	/* restore original value */
					show_author = var_orig;
					switch (show_author) {
						case SHOW_FROM_NONE:
							str = txt_show_from_none;
							break;
						case SHOW_FROM_ADDR:
							str = txt_show_from_addr;
							break;
						case SHOW_FROM_NAME:
							str = txt_show_from_name;
							break;
						case SHOW_FROM_BOTH:
							str = txt_show_from_both;
							break;
					}
					printf ("%s", str);
					fflush (stdout);
				} else {
					default_show_author = show_author;	
					if (show_author == SHOW_FROM_BOTH) {
						max_subj = (COLS / 2) - 2;
					} else {
						max_subj = (COLS / 2) + 5;
					}
					max_from = (COLS - max_subj) - 17;
				}
				break;

			case 14:
				var_orig = post_proc_type;
				show_menu_help (txt_help_post_proc_type);
				do {
					MoveCursor (INDEX_TOP+8, COL2 + (int) strlen (txt_opt_process_type));
					if ((ch	= ReadCh()) == ' ') {
						if (post_proc_type + 1 > POST_PROC_UUD_EXT_ZOO) {
							post_proc_type = POST_PROC_NONE;
						} else {
							post_proc_type++;
						}
						switch (post_proc_type) {
							case POST_PROC_NONE:
								str = txt_post_process_none;
								proc_ch_default = 'n';
								break;
							case POST_PROC_SHAR:
								str = txt_post_process_sh;
								proc_ch_default = 's';
								break;
							case POST_PROC_UUDECODE:
								str = txt_post_process_uudecode;
								proc_ch_default = 'u';
								break;
							case POST_PROC_UUD_LST_ZOO:
								str = txt_post_process_uud_lst_zoo;
								proc_ch_default = 'U';
								break;
							case POST_PROC_UUD_EXT_ZOO:
								str = txt_post_process_uud_ext_zoo;
								proc_ch_default = 'U';
								break;
						}
						CleartoEOLN (); 
						printf ("%s", str);
						fflush (stdout);
					}
				} while (ch != CR && ch != ESC);

				if (ch == ESC) {	/* restore original value */
					post_proc_type = var_orig;
					switch (post_proc_type) {
						case POST_PROC_NONE:
							str = txt_post_process_none;
							proc_ch_default = 'n';
							break;
						case POST_PROC_SHAR:
							str = txt_post_process_sh;
							proc_ch_default = 's';
							break;
						case POST_PROC_UUDECODE:
							str = txt_post_process_uudecode;
							proc_ch_default = 'u';
							break;
						case POST_PROC_UUD_LST_ZOO:
							str = txt_post_process_uud_lst_zoo;
							proc_ch_default = 'U';
							break;
						case POST_PROC_UUD_EXT_ZOO:
							str = txt_post_process_uud_ext_zoo;
							proc_ch_default = 'U';
							break;
					}
					CleartoEOLN (); 
					printf ("%s", str);
					fflush (stdout);
				}
				break;

			case 15:
				var_orig = sort_art_type;
				show_menu_help (txt_help_sort_type);
				do {
					MoveCursor (INDEX_TOP+10, COL1 + (int) strlen (txt_opt_sort_type));
					if ((ch	= ReadCh()) == ' ') {
						if (sort_art_type + 1 > SORT_BY_DATE_ASCEND) {
							sort_art_type = SORT_BY_NOTHING;
						} else {
							sort_art_type++;
						}
						switch (sort_art_type) {
							case SORT_BY_NOTHING:
								str = txt_sort_by_nothing;
								break;
							case SORT_BY_SUBJ_DESCEND:
								str = txt_sort_by_subj_descend;
								break;
							case SORT_BY_SUBJ_ASCEND:
								str = txt_sort_by_subj_ascend;
								break;
							case SORT_BY_FROM_DESCEND:
								str = txt_sort_by_from_descend;
								break;
							case SORT_BY_FROM_ASCEND:
								str = txt_sort_by_from_ascend;
								break;
							case SORT_BY_DATE_DESCEND:
								str = txt_sort_by_date_descend;
								break;
							case SORT_BY_DATE_ASCEND:
								str = txt_sort_by_date_ascend;
								break;
						}
						CleartoEOLN (); 
						printf ("%s", str);
						fflush (stdout);
					}
				} while (ch != CR && ch != ESC);

				if (ch == ESC) {	/* restore original value */
					sort_art_type = var_orig;
					switch (sort_art_type) {
						case SORT_BY_NOTHING:
							str = txt_sort_by_nothing;
							break;
						case SORT_BY_SUBJ_DESCEND:
							str = txt_sort_by_subj_descend;
							break;
						case SORT_BY_SUBJ_ASCEND:
							str = txt_sort_by_subj_ascend;
							break;
						case SORT_BY_FROM_DESCEND:
							str = txt_sort_by_from_descend;
							break;
						case SORT_BY_FROM_ASCEND:
							str = txt_sort_by_from_ascend;
							break;
						case SORT_BY_DATE_DESCEND:
							str = txt_sort_by_date_descend;
							break;
						case SORT_BY_DATE_ASCEND:
							str = txt_sort_by_date_ascend;
							break;
					}
					CleartoEOLN (); 
					printf ("%s", str);
					fflush (stdout);
				}
				break;

			case 16:
				show_menu_help (txt_help_savedir);
				prompt_menu_string (INDEX_TOP+12, COL1 + (int) strlen (txt_opt_savedir), savedir);
				expand_rel_abs_pathname (INDEX_TOP+12, COL1 + (int) strlen (txt_opt_savedir), savedir);
				break;

			case 17:
				show_menu_help (txt_help_maildir);
				prompt_menu_string (INDEX_TOP+14, COL1 + (int) strlen (txt_opt_maildir), maildir);
				expand_rel_abs_pathname (INDEX_TOP+14, COL1 + (int) strlen (txt_opt_maildir), maildir);
				break;

			case 18:
				show_menu_help (txt_help_printer);
				prompt_menu_string (INDEX_TOP+16, COL1 + (int) strlen (txt_opt_printer), printer);
				expand_rel_abs_pathname (INDEX_TOP+16, COL1 + (int) strlen (txt_opt_printer), printer);
				break;
		}
		show_menu_help (txt_select_rcfile_option);
	}
}


void show_rcfile_menu ()
{
	char *str;

	ClearScreen ();

	center_line (0, TRUE, txt_options_menu);
	
	MoveCursor (INDEX_TOP, 0);
	printf ("%s%s\r\n\r\n", txt_opt_autosave, (save_archive_name ? "ON " : "OFF"));
	printf ("%s%s\r\n\r\n", txt_opt_kill_articles, (kill_articles ? "ON " : "OFF"));
	printf ("%s%s\r\n\r\n", txt_opt_pos_first_unread, (pos_first_unread ? "ON " : "OFF"));
	printf ("%s%s", txt_opt_thread_arts, (thread_arts ? "ON " : "OFF"));

	MoveCursor(INDEX_TOP, COL2);
	printf ("%s%s", txt_opt_save_separate, (save_separate ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+2, COL2);
	printf ("%s%s", txt_opt_draw_arrow, (draw_arrow_mark ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+4, COL2);
	printf ("%s%s", txt_opt_page_scroll, (full_page_scroll ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+6, COL2);
	printf ("%s%s", txt_opt_show_only_unread, (show_only_unread ? "ON " : "OFF"));

	MoveCursor(INDEX_TOP, COL3);
	printf ("%s%s", txt_opt_mark_saved_read, (mark_saved_read ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+2, COL3);
	printf ("%s%s", txt_opt_print_header, (print_header ? "ON " : "OFF"));
	MoveCursor(INDEX_TOP+4, COL3);
	printf ("%s%s", txt_opt_catchup_groups, (catchup_read_groups ? "ON " : "OFF"));

	MoveCursor(INDEX_TOP+8, COL1);
	switch (show_author) {
		case SHOW_FROM_NONE:
			str = txt_show_from_none;
			break;
		case SHOW_FROM_ADDR:
			str = txt_show_from_addr;
			break;
		case SHOW_FROM_NAME:
			str = txt_show_from_name;
			break;
		case SHOW_FROM_BOTH:
			str = txt_show_from_both;
			break;
		}
	printf ("%s%s", txt_opt_show_author, str);
	MoveCursor(INDEX_TOP+8, COL2);
	switch (post_proc_type) {
		case POST_PROC_NONE:
			str = txt_post_process_none;
			break;
		case POST_PROC_SHAR:
			str = txt_post_process_sh;
			break;
		case POST_PROC_UUDECODE:
			str = txt_post_process_uudecode;
			break;
		case POST_PROC_UUD_LST_ZOO:
			str = txt_post_process_uud_lst_zoo;
			break;
		case POST_PROC_UUD_EXT_ZOO:
			str = txt_post_process_uud_ext_zoo;
			break;
	}
	printf ("%s%s\r\n\r\n", txt_opt_process_type, str);
	
	MoveCursor(INDEX_TOP+10, COL1);
	switch (sort_art_type) {
		case SORT_BY_NOTHING:
			str = txt_sort_by_nothing;
			break;
		case SORT_BY_SUBJ_DESCEND:
			str = txt_sort_by_subj_descend;
			break;
		case SORT_BY_SUBJ_ASCEND:
			str = txt_sort_by_subj_ascend;
			break;
		case SORT_BY_FROM_DESCEND:
			str = txt_sort_by_from_descend;
			break;
		case SORT_BY_FROM_ASCEND:
			str = txt_sort_by_from_ascend;
			break;
		case SORT_BY_DATE_DESCEND:
			str = txt_sort_by_date_descend;
			break;
		case SORT_BY_DATE_ASCEND:
			str = txt_sort_by_date_ascend;
			break;
	}
	printf ("%s%s\r\n\r\n", txt_opt_sort_type, str);

	printf ("%s%s\r\n\r\n", txt_opt_savedir, savedir);
	printf ("%s%s\r\n\r\n", txt_opt_maildir, maildir);
	printf ("%s%s\r\n\r\n", txt_opt_printer, printer);
	fflush(stdout);

	show_menu_help (txt_select_rcfile_option);
	MoveCursor (LINES, 0);
}

/*
 *  expand ~/News to /usr/username/News and print to screen
 */
 
void expand_rel_abs_pathname (line, col, str)
	int line;
	int col;
	char *str;
{
	char buf[LEN];
	
	if (str[0] == '~') {
		if (strlen (str) == 1) {
			strcpy (str, homedir);
		} else {
			sprintf (buf, "%s%s", homedir, str+1);
			strcpy (str, buf);
		}
	}
	MoveCursor (line, col);
	CleartoEOLN ();
	puts (str);
	fflush (stdout);
}

/*
 *  show_menu_help
 */
 
void show_menu_help (help_message)
	char *help_message;
{
	 MoveCursor (LINES-2, 0);
	 CleartoEOLN ();
	 center_line (LINES-2, FALSE, help_message);
}
