/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : thread.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 08-02-92
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern int index_point;
int threaded_on_subject;
static int top_thread = 0;
static int thread_index_point = 0;
static int thread_basenote = 0;
static int thread_respnum = 0;
static int first_thread_on_screen = 0;
static int last_thread_on_screen = 0;


/*
 * show current thread. If threaded on Subject: show
 *   <respnum> <name>    <respnum> <name>
 * If threaded on Archive-name: show
 *   <respnum> <subject> <name>
 */
 
int show_thread (respnum, group, group_path)
	int respnum;
	char *group;
	char *group_path;
{
	int ch;
	int flag, i, index, n;
	int scroll_lines;
	int	thread_marked_unread;

	thread_respnum = respnum;
	thread_basenote = which_thread (thread_respnum);
	top_thread = num_of_responses (thread_basenote);

	if (debug) {
		sprintf (msg, "THREAD respnum=[%d]  basenote=[%d]  num=[%d]",
			thread_respnum, thread_basenote, top_thread);
		info_message (msg);
		sleep (2);
	}

	if (top_thread <= 0) {
		info_message (txt_no_resps_in_thread);
		return FALSE;
	}

	if (arts[thread_respnum].archive != (char *) 0) {
		threaded_on_subject = FALSE;
	} else {
		threaded_on_subject = TRUE;
	}

	if (space_mode) {
		if (i = new_responses (thread_basenote)) {
			for (n=0, i = thread_basenote; i >= 0 ; i = arts[i].thread, n++) {
				if (arts[i].unread == ART_UNREAD) {
					thread_index_point = n;
					break;
				}
			}
		} else {
			thread_index_point = top_thread;
		}
	} else {
		thread_index_point = top_thread;
	}

	if (thread_index_point < 0) {
		thread_index_point = 0;
	}

	show_thread_page ();

	while (TRUE) {
		ch = (char) ReadCh();

		if (ch >= '0' && ch <= '9') {	/* 0 goes to basenote */
			prompt_thread_num (ch);
		} else switch (ch) {
			case ESC:	/* common arrow keys */
				switch (get_arrow_key ()) {
					case KEYMAP_UP:
						goto thread_up;

					case KEYMAP_DOWN:
						goto thread_down;

					case KEYMAP_PAGE_UP:
						goto thread_page_up;

					case KEYMAP_PAGE_DOWN:
						goto thread_page_down;

					case KEYMAP_HOME:
						if (thread_index_point != 0) {
							thread_index_point = 0;
							show_thread_page ();
						}
						break;
					
					case KEYMAP_END:
						goto end_of_thread;
				}
				break;

			case '$':	/* show last page of threads */
end_of_thread:			
				if (thread_index_point != top_thread) {
					thread_index_point = top_thread;
					show_thread_page ();
				}
				break;
				
			case '\r':
			case '\n':	/* read current article within thread */
				n = choose_response (thread_basenote, thread_index_point);
				n = show_page (n, group, group_path);
				if (n == thread_basenote) {
					show_thread_page ();
				} else {
					index_point = n;	
					goto thread_done;
				}
				break;

			case '\t':
 				space_mode = TRUE;
				if (thread_index_point == 0) {
					n = thread_respnum;
				} else {
					n = choose_response (thread_basenote, thread_index_point);
				}
				index = thread_index_point;
				for (i = n ; i != -1 ; i = arts[i].thread) {
					if (arts[i].unread == ART_UNREAD) {
						n = show_page (i, group, group_path);
						break;
					}
					index++;
				}
				if (n == thread_basenote) {
					thread_index_point = index;
					show_thread_page ();
				} else {
					index_point = which_thread (n);	
					goto thread_done;
				}
				break;
	
			case ' ':			/* page down */
			case ctrl('D'):		/* vi style */
			case ctrl('V'):		/* emacs style */
thread_page_down:
				if (thread_index_point == top_thread)
					break;

				erase_thread_arrow ();
				scroll_lines = (full_page_scroll ? NOTESLINES : NOTESLINES / 2);
				thread_index_point = ((thread_index_point + scroll_lines) /
									scroll_lines) * scroll_lines;
				if (thread_index_point > top_thread) {
					thread_index_point = (top_thread / scroll_lines) * scroll_lines;
					if (thread_index_point < top_thread) {
						thread_index_point = top_thread;
					}
				}
				if (thread_index_point < first_thread_on_screen ||
					thread_index_point >= last_thread_on_screen) {
					show_thread_page ();
				} else {
					draw_thread_arrow ();
				}
				break;

			case ctrl('L'):		/* redraw screen */
			case ctrl('R'):
			case ctrl('W'):
#ifndef USE_CLEARSCREEN
				ClearScreen ();
#endif
				show_thread_page ();
				break;

			case ctrl('N'):
			case 'j':		/* line down */
thread_down:
				if (thread_index_point + 1 > top_thread)
					break;
/*
				if (thread_index_point + 1 >= top_thread)
					break;
*/

				if (thread_index_point + 1 > last_thread_on_screen) {
/*
				if (thread_index_point + 1 >= last_thread_on_screen) {
*/
#ifndef USE_CLEARSCREEN
					erase_thread_arrow ();
#endif					
					thread_index_point++;
					show_thread_page ();
				} else {
					erase_thread_arrow ();
					thread_index_point++;
					draw_thread_arrow ();
				}
				break;

			case ctrl('P'):
			case 'k':		/* line up */
thread_up:
				if (! thread_index_point)
					break;

				if (thread_index_point <= first_thread_on_screen) {
					thread_index_point--;
					show_thread_page ();
				} else {
					erase_thread_arrow ();
					thread_index_point--;
					draw_thread_arrow ();
				}
				break;

			case ctrl('U'):		/* page up */
			case 'b':
thread_page_up:
#ifndef USE_CLEARSCREEN
				clear_message ();
#endif
				erase_thread_arrow ();
				scroll_lines = (full_page_scroll ? NOTESLINES : NOTESLINES / 2);
				if ((n = thread_index_point % scroll_lines) > 0) {
					thread_index_point = thread_index_point - n;
				} else {
					thread_index_point = ((thread_index_point - scroll_lines) / scroll_lines) * scroll_lines;
				}
				if (thread_index_point < 0) {
					thread_index_point = 0;
				}
				if (thread_index_point < first_thread_on_screen
				|| thread_index_point >= last_thread_on_screen)
					show_thread_page ();
				else
					draw_thread_arrow ();
				break;

			case 'h':	/* help */
				show_info_page (HELP_INFO, help_thread, txt_thread_com);
				show_thread_page ();
				break;

			case 'I':	/* toggle inverse video */
				toggle_inverse_video ();
				show_thread_page ();
				break;

			case 'i':	/* return */
			case 'q':
			case 't':
				goto thread_done;

			case 'K':	/* mark thread as read */
				for (i = (int) base[thread_basenote] ; i != -1 ; i = arts[i].thread) {
					arts[i].unread = ART_READ;
				}
				goto thread_done;
				break;

			case 'v':	/* version */
				info_message (cvers);
				break;

			case 'z':	/* mark article as unread */
				if (thread_index_point == 0) {
					n = thread_respnum;
				} else {
					n = choose_response (thread_basenote, thread_index_point);
				}
				if (n >= 0) {
					arts[n].unread = ART_UNREAD;
					show_thread_page ();
					info_message (txt_art_marked_as_unread);
				}
				break;

			case 'Z':	/* mark thread as unread */
				for (i = (int) base[thread_basenote] ; i != -1 ; i = arts[i].thread) {
					arts[i].unread = ART_UNREAD;
				}
				show_thread_page ();
				info_message (txt_thread_marked_as_unread);
				break;
				
			default:
			    info_message (txt_bad_command);
		}
	}

thread_done:
/*
	fix_new_highest (sav_groupnum);
	update_newsrc (group, my_group[sav_groupnum], FALSE);
*/
	clear_note_area ();
	return TRUE;
}


void show_thread_page ()
{
	extern int index_point;
	char buf[LEN];
	char new_resps[8];
	char resps[8];
	char from[LEN];
	char subject[LEN];
	int col, i, j, n;
	static int index = 0;
	int len_from;
	int len_subj;

	set_signals_thread ();
	
	ClearScreen ();

	if (threaded_on_subject) {
		sprintf (msg, "Thread (%s)", arts[thread_respnum].subject);
	} else {
		sprintf (msg, "List Thread (%d of %d)", index_point+1, top_base);
	}
	show_title (msg);

	MoveCursor (INDEX_TOP, 0);
	if (thread_index_point > top_thread) {
		thread_index_point = top_thread - 1;
	}

	if (NOTESLINES <= 0) {
		first_thread_on_screen = 0;
	} else {
		first_thread_on_screen = (thread_index_point / NOTESLINES) * NOTESLINES;
		if (first_thread_on_screen < 0) {
			first_thread_on_screen = 0;
		}
	}

	last_thread_on_screen = first_thread_on_screen + NOTESLINES;

	if (last_thread_on_screen >= top_thread) {
		last_thread_on_screen = top_thread;
		first_thread_on_screen = (top_thread / NOTESLINES) * NOTESLINES;

		if (first_thread_on_screen == last_thread_on_screen ||
			first_thread_on_screen < 0) {
			if (first_thread_on_screen < 0) {
				first_thread_on_screen = 0;
			} else {
				first_thread_on_screen = last_thread_on_screen - NOTESLINES;
			}
		}
	}

	if (top_thread == 0) {
		first_thread_on_screen = 0;
		last_thread_on_screen = 0;
	}

	index = choose_response (thread_basenote, thread_index_point);

if (debug) {
sprintf (msg, "first=[%d] last=[%d] index=[%d] thread_index_point=[%d]",
	first_thread_on_screen, last_thread_on_screen,
	index, thread_index_point);
info_message (msg);
sleep (2);
MoveCursor (INDEX_TOP, 0);
}

	for (j=0, i = first_thread_on_screen; j < NOTESLINES && i <= last_thread_on_screen; i++, j++) {
		if (i == 0) {
			index = thread_respnum;
		} else {
			if ((index = next_response (index)) == -1) {
				break;
			}
		}

		if (arts[index].tagged) {
			sprintf (new_resps, "%3d", arts[index].tagged);
		} else if (arts[index].unread == ART_UNREAD) {
			sprintf (new_resps, "  %c", UNREAD_ART_MARK);
		} else {
			strcpy (new_resps, "   ");
		}

		get_author (TRUE, index, from);

		if (draw_arrow_mark) {
			if (threaded_on_subject) {
				printf ("  %4d%3s  %s\r\n", i, new_resps, from);
			} else {
				my_strncpy (subject, arts[index].subject, max_subj);
				printf ("  %4d%3s  %-*s   %-*s\r\n",
					i, new_resps, max_subj, subject, max_from, from);
			}
		} else {
			if (threaded_on_subject) {
				sprintf (screen[j].col, "  %4d%3s  %-*s\r\n", i, new_resps, max_subj+max_from+5, from);
			} else {
				my_strncpy (subject, arts[index].subject, max_subj+2);
				sprintf (screen[j].col, "  %4d%3s  %-*s   %-*s\r\n",
					i, new_resps, max_subj+2, subject, max_from, from);
			}
			printf ("%s", screen[j].col);
		}
	}

#ifndef USE_CLEARSCREEN
	CleartoEOS ();
#endif

	if (last_thread_on_screen == top_thread) {
		info_message (txt_end_of_thread);
	}

	draw_thread_arrow ();
}


void draw_thread_arrow ()
{
	draw_arrow (INDEX_TOP + (thread_index_point-first_thread_on_screen));
}


void erase_thread_arrow ()
{
	erase_arrow (INDEX_TOP + (thread_index_point-first_thread_on_screen));
}


int prompt_thread_num (ch)
	char ch;
{
	int num;

	clear_message ();

	if ((num = prompt_num (ch, txt_read_art)) == -1) {
		clear_message ();
		return FALSE;
	}

	if (num >= top_thread)
		num = top_thread - 1;

	if (num >= first_thread_on_screen
	&&  num < last_thread_on_screen) {
		erase_thread_arrow ();
		thread_index_point = num;
		draw_thread_arrow ();
	} else {
#ifndef USE_CLEARSCREEN
		erase_thread_arrow ();
#endif		
		thread_index_point = num;
		show_thread_page ();
	}
	return TRUE;
}

/*
 *  Return the number of unread articles there are within a thread
 */

int new_responses (thread)
	int thread;
{
	int i;
	int sum = 0;

	for (i = (int) base[thread]; i >= 0; i = arts[i].thread) {
		if (arts[i].unread) {
			sum++;
		}
	}
	
	return sum;
}

/*
 *  Which base note (an index into base[]) does a respnum
 *  (an index into arts[]) corresponsd to?
 *
 *  In other words, base[] points to an entry in arts[] which is
 *  the head of a thread, linked with arts[].thread.  For any q: arts[q],
 *  find i such that base[i]->arts[n]->arts[o]->...->arts[q]
 */

int which_thread (n)
	int n;
{
	register int i, j;

	for (i = 0; i < top_base; i++) {
		for (j = (int) base[i] ; j >= 0 ; j = arts[j].thread) {
			if (j == n) {
				return i;
			}
		}
	}

	sprintf (msg, "%d", n);
	error_message (txt_cannot_find_base_art, msg);
	return -1;
}

/*
 *  Find how deep in a thread a response is.  Start counting at zero
 */

int which_response (n)
	int n;
{
	int i, j;
	int num = 0;

	i = which_thread (n);

	for (j = (int) base[i]; j != -1; j = arts[j].thread)
		if (j == n)
			break;
		else
			num++;

	return num;
}

/*
 *  Given an index into base[], find the number of responses for
 *  that basenote
 */

int num_of_responses (n)
	int n;
{
	int i;
	int oldi = -3;
	int sum = 0;

	assert (n < top_base);

	for (i = (int) base[n]; i != -1; i = arts[i].thread) {
		assert (i != -2);
		assert (i != oldi);
		oldi = i;
		sum++;
	}

	return sum - 1;
}

/*
 *  Find the next response.  Go to the next basenote if there
 *  are no more responses in this thread
 */

int next_response (n)
	int n;
{
	int i;

	if (arts[n].thread >= 0)
		return arts[n].thread;

	i = which_thread (n) + 1;

	if (i >= top_base)
		return -1;

	return (int) base[i];
}

/*
 *  Given a respnum (index into arts[]), find the respnum of the
 *  next basenote
 */

int next_thread (n)
	int n;
{
	int i;

	i = which_thread (n) + 1;
	if (i >= top_base)
		return -1;

	return (int) base[i];
}

/*
 *  Find the previous response.  Go to the last response in the previous
 *  thread if we go past the beginning of this thread.
 */

int prev_response (n)
	int n;
{
	int resp;
	int i;

	resp = which_response (n);

	if (resp > 0)
		return choose_response (which_thread (n), resp-1);

	i = which_thread (n) - 1;

	if (i < 0)
		return -1;

	return choose_response (i, num_of_responses (i));
}

/*
 *  return response number n from thread i
 */

int choose_response (i, n)
	int i;
	int n;
{
	int j;

	j = (int) base[i];

	while (n-- && arts[j].thread >= 0) {
		j = arts[j].thread;
	}

	return j;
}

/*
 *  Find the next unread response in this group 
 */

int next_unread (n)
	int n;
{
	while (n >= 0) {
		if (arts[n].unread == ART_UNREAD) {
			return n;
		}
		n = next_response (n);
	}

	return -1;
}


/*
 *  Find the previous unread response in this thread
 */

int prev_unread (n)
	int n;
{
	while (n >= 0) {
		if (arts[n].unread == ART_UNREAD) {
			return n;
		}
		n = prev_response (n);
	}

	return -1;
}
